@@ -28,7 +28,7 @@
 #include <string.h>
 #include <cairo-xlib-xrender.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 
 static CompMetadata annoMetadata;
 
@@ -41,11 +41,12 @@ static int annoLastPointerY = 0;
 #define ANNO_DISPLAY_OPTION_DRAW_BUTTON	    1
 #define ANNO_DISPLAY_OPTION_ERASE_BUTTON    2
 #define ANNO_DISPLAY_OPTION_CLEAR_KEY       3
-#define ANNO_DISPLAY_OPTION_FILL_COLOR      4
-#define ANNO_DISPLAY_OPTION_STROKE_COLOR    5
-#define ANNO_DISPLAY_OPTION_LINE_WIDTH      6
-#define ANNO_DISPLAY_OPTION_STROKE_WIDTH    7
-#define ANNO_DISPLAY_OPTION_NUM	            8
+#define ANNO_DISPLAY_OPTION_CLEAR_BUTTON    4
+#define ANNO_DISPLAY_OPTION_FILL_COLOR      5
+#define ANNO_DISPLAY_OPTION_STROKE_COLOR    6
+#define ANNO_DISPLAY_OPTION_LINE_WIDTH      7
+#define ANNO_DISPLAY_OPTION_STROKE_WIDTH    8
+#define ANNO_DISPLAY_OPTION_NUM	            9
 
 typedef struct _AnnoDisplay {
     int		    screenPrivateIndex;
@@ -67,14 +68,14 @@ typedef struct _AnnoScreen {
     Bool eraseMode;
 } AnnoScreen;
 
-#define GET_ANNO_DISPLAY(d)				     \
-    ((AnnoDisplay *) (d)->privates[displayPrivateIndex].ptr)
+#define GET_ANNO_DISPLAY(d)					  \
+    ((AnnoDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 
 #define ANNO_DISPLAY(d)			   \
     AnnoDisplay *ad = GET_ANNO_DISPLAY (d)
 
-#define GET_ANNO_SCREEN(s, ad)					 \
-    ((AnnoScreen *) (s)->privates[(ad)->screenPrivateIndex].ptr)
+#define GET_ANNO_SCREEN(s, ad)					      \
+    ((AnnoScreen *) (s)->base.privates[(ad)->screenPrivateIndex].ptr)
 
 #define ANNO_SCREEN(s)							\
     AnnoScreen *as = GET_ANNO_SCREEN (s, GET_ANNO_DISPLAY (s->display))
@@ -743,6 +744,7 @@ static const CompMetadataOptionInfo annoDisplayOptionInfo[] = {
     { "draw_button", "button", 0, annoDraw, 0 },
     { "erase_button", "button", 0, annoEraseInitiate, 0 },
     { "clear_key", "key", 0, annoClear, 0 },
+    { "clear_button", "button", 0, annoClear, 0 },
     { "fill_color", "color", 0, 0, 0 },
     { "stroke_color", "color", 0, 0, 0 },
     { "line_width", "float", 0, 0, 0 },
@@ -755,6 +757,9 @@ annoInitDisplay (CompPlugin  *p,
 {
     AnnoDisplay *ad;
 
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
+	return FALSE;
+
     ad = malloc (sizeof (AnnoDisplay));
     if (!ad)
 	return FALSE;
@@ -779,7 +784,7 @@ annoInitDisplay (CompPlugin  *p,
 
     WRAP (ad, d, handleEvent, annoHandleEvent);
 
-    d->privates[displayPrivateIndex].ptr = ad;
+    d->base.privates[displayPrivateIndex].ptr = ad;
 
     return TRUE;
 }
@@ -821,7 +826,7 @@ annoInitScreen (CompPlugin *p,
 
     WRAP (as, s, paintOutput, annoPaintOutput);
 
-    s->privates[ad->screenPrivateIndex].ptr = as;
+    s->base.privates[ad->screenPrivateIndex].ptr = as;
 
     return TRUE;
 }
@@ -848,6 +853,61 @@ annoFiniScreen (CompPlugin *p,
     free (as);
 }
 
+static CompBool
+annoInitObject (CompPlugin *p,
+		CompObject *o)
+{
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) 0, /* InitCore */
+	(InitPluginObjectProc) annoInitDisplay,
+	(InitPluginObjectProc) annoInitScreen
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+annoFiniObject (CompPlugin *p,
+		CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) 0, /* FiniCore */
+	(FiniPluginObjectProc) annoFiniDisplay,
+	(FiniPluginObjectProc) annoFiniScreen
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
+}
+
+static CompOption *
+annoGetObjectOptions (CompPlugin *plugin,
+		      CompObject *object,
+		      int	 *count)
+{
+    static GetPluginObjectOptionsProc dispTab[] = {
+	(GetPluginObjectOptionsProc) 0, /* GetCoreOptions */
+	(GetPluginObjectOptionsProc) annoGetDisplayOptions
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
+		     (void *) (*count = 0), (plugin, object, count));
+}
+
+static CompBool
+annoSetObjectOption (CompPlugin      *plugin,
+		     CompObject      *object,
+		     const char      *name,
+		     CompOptionValue *value)
+{
+    static SetPluginObjectOptionProc dispTab[] = {
+	(SetPluginObjectOptionProc) 0, /* SetCoreOption */
+	(SetPluginObjectOptionProc) annoSetDisplayOption
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab), FALSE,
+		     (plugin, object, name, value));
+}
+
 static Bool
 annoInit (CompPlugin *p)
 {
@@ -877,13 +937,6 @@ annoFini (CompPlugin *p)
     compFiniMetadata (&annoMetadata);
 }
 
-static int
-annoGetVersion (CompPlugin *plugin,
-		int	   version)
-{
-    return ABIVERSION;
-}
-
 static CompMetadata *
 annoGetMetadata (CompPlugin *plugin)
 {
@@ -892,24 +945,17 @@ annoGetMetadata (CompPlugin *plugin)
 
 static CompPluginVTable annoVTable = {
     "annotate",
-    annoGetVersion,
     annoGetMetadata,
     annoInit,
     annoFini,
-    annoInitDisplay,
-    annoFiniDisplay,
-    annoInitScreen,
-    annoFiniScreen,
-    0, /* InitWindow */
-    0, /* FiniWindow */
-    annoGetDisplayOptions,
-    annoSetDisplayOption,
-    0, /* GetScreenOptions */
-    0  /* SetScreenOption */
+    annoInitObject,
+    annoFiniObject,
+    annoGetObjectOptions,
+    annoSetObjectOption
 };
 
 CompPluginVTable *
-getCompPluginInfo (void)
+getCompPluginInfo20070830 (void)
 {
     return &annoVTable;
 }
