@@ -23,7 +23,7 @@
 #include <ksimpleconfig.h>
 #include <qfile.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 
 #define COMPIZ_KCONFIG_RC "compizrc"
 
@@ -31,40 +31,25 @@ static KInstance *kInstance;
 
 static CompMetadata kconfigMetadata;
 
-static int displayPrivateIndex;
-
-typedef struct _KconfigDisplay {
-    int screenPrivateIndex;
-
-    InitPluginForDisplayProc      initPluginForDisplay;
-    SetDisplayOptionProc	  setDisplayOption;
-    SetDisplayOptionForPluginProc setDisplayOptionForPlugin;
+static int corePrivateIndex;
 
+typedef struct _KconfigCore {
     KConfig *config;
 
     CompTimeoutHandle   syncHandle;
     CompTimeoutHandle   reloadHandle;
     CompFileWatchHandle fileWatch;
-} KconfigDisplay;
 
-typedef struct _KconfigScreen {
-    InitPluginForScreenProc      initPluginForScreen;
-    SetScreenOptionProc		 setScreenOption;
-    SetScreenOptionForPluginProc setScreenOptionForPlugin;
-} KconfigScreen;
+    InitPluginForObjectProc initPluginForObject;
+    SetOptionForPluginProc  setOptionForPlugin;
+} KconfigCore;
 
-#define GET_KCONFIG_DISPLAY(d)				        \
-    ((KconfigDisplay *) (d)->privates[displayPrivateIndex].ptr)
+#define GET_KCONFIG_CORE(c)				       \
+    ((KconfigCore *) (c)->base.privates[corePrivateIndex].ptr)
 
-#define KCONFIG_DISPLAY(d)		         \
-    KconfigDisplay *kd = GET_KCONFIG_DISPLAY (d)
+#define KCONFIG_CORE(c)			   \
+    KconfigCore *kc = GET_KCONFIG_CORE (c)
 
-#define GET_KCONFIG_SCREEN(s, kd)				    \
-    ((KconfigScreen *) (s)->privates[(kd)->screenPrivateIndex].ptr)
-
-#define KCONFIG_SCREEN(s)					              \
-    KconfigScreen *ks = GET_KCONFIG_SCREEN (s,				      \
-					    GET_KCONFIG_DISPLAY (s->display))
 
 static void
 kconfigRcChanged (const char *name,
@@ -73,13 +58,11 @@ kconfigRcChanged (const char *name,
 static Bool
 kconfigRcSync (void *closure)
 {
-    CompDisplay *d = (CompDisplay *) closure;
+    KCONFIG_CORE (&core);
 
-    KCONFIG_DISPLAY (d);
+    kc->config->sync ();
 
-    kd->config->sync ();
-
-    kd->syncHandle = 0;
+    kc->syncHandle = 0;
 
     return FALSE;
 }
@@ -101,7 +84,7 @@ kconfigValueToBool (CompOptionType  type,
 }
 
 static QString
-kconfigValueToString (CompDisplay     *d,
+kconfigValueToString (CompObject      *object,
 		      CompOptionType  type,
 		      CompOptionValue *value)
 {
@@ -130,7 +113,10 @@ kconfigValueToString (CompDisplay     *d,
     case CompOptionTypeKey: {
 	char *action;
 
-	action = keyActionToString (d, &value->action);
+	while (object && object->type != COMP_OBJECT_TYPE_DISPLAY)
+	    object = object->parent;
+
+	action = keyActionToString (GET_CORE_DISPLAY (object), &value->action);
 	if (action)
 	{
 	    str = QString (action);
@@ -140,7 +126,11 @@ kconfigValueToString (CompDisplay     *d,
     case CompOptionTypeButton: {
 	char *action;
 
-	action = buttonActionToString (d, &value->action);
+	while (object && object->type != COMP_OBJECT_TYPE_DISPLAY)
+	    object = object->parent;
+
+	action = buttonActionToString (GET_CORE_DISPLAY (object),
+				       &value->action);
 	if (action)
 	{
 	    str = QString (action);
@@ -177,29 +167,44 @@ kconfigValueToString (CompDisplay     *d,
     return str;
 }
 
-static void
-kconfigSetOption (CompDisplay *d,
-		  CompOption  *o,
-		  const char  *plugin,
-		  const char  *object)
+static QString
+kconfigObjectString (CompObject *object)
 {
-    QString group (QString (plugin) + "_" + QString (object));
+    QString objectName (QString (compObjectTypeName (object->type)));
+    char    *name;
 
-    KCONFIG_DISPLAY (d);
+    name = compObjectName (object);
+    if (name)
+    {
+	objectName += name;
+	free (name);
+    }
 
-    kd->config->setGroup (group);
+    return objectName;
+}
+
+static void
+kconfigSetOption (CompObject *object,
+		  CompOption *o,
+		  const char *plugin)
+{
+    QString group (QString (plugin) + "_" + kconfigObjectString (object));
+
+    KCONFIG_CORE (&core);
+
+    kc->config->setGroup (group);
 
     switch (o->type) {
     case CompOptionTypeBool:
     case CompOptionTypeBell:
-	kd->config->writeEntry (o->name,
+	kc->config->writeEntry (o->name,
 				kconfigValueToBool (o->type, &o->value));
 	break;
     case CompOptionTypeInt:
-	kd->config->writeEntry (o->name, o->value.i);
+	kc->config->writeEntry (o->name, o->value.i);
 	break;
     case CompOptionTypeFloat:
-	kd->config->writeEntry (o->name, (double) o->value.f);
+	kc->config->writeEntry (o->name, (double) o->value.f);
 	break;
     case CompOptionTypeString:
     case CompOptionTypeColor:
@@ -207,8 +212,9 @@ kconfigSetOption (CompDisplay *d,
     case CompOptionTypeButton:
     case CompOptionTypeEdge:
     case CompOptionTypeMatch:
-	kd->config->writeEntry (o->name,
-				kconfigValueToString (d, o->type, &o->value));
+	kc->config->writeEntry (o->name,
+				kconfigValueToString (object, o->type,
+						      &o->value));
 	break;
     case CompOptionTypeList: {
 	int i;
@@ -220,7 +226,7 @@ kconfigSetOption (CompDisplay *d,
 	    for (i = 0; i < o->value.list.nValue; i++)
 		list += o->value.list.value[i].i;
 
-	    kd->config->writeEntry (o->name, list);
+	    kc->config->writeEntry (o->name, list);
 	} break;
 	case CompOptionTypeBool:
 	case CompOptionTypeFloat:
@@ -234,11 +240,11 @@ kconfigSetOption (CompDisplay *d,
 	    QStringList list;
 
 	    for (i = 0; i < o->value.list.nValue; i++)
-		list += kconfigValueToString (d,
+		list += kconfigValueToString (object,
 					      o->value.list.type,
 					      &o->value.list.value[i]);
 
-	    kd->config->writeEntry (o->name, list);
+	    kc->config->writeEntry (o->name, list);
 	} break;
 	case CompOptionTypeAction:
 	case CompOptionTypeList:
@@ -249,12 +255,12 @@ kconfigSetOption (CompDisplay *d,
 	return;
     }
 
-    if (!kd->syncHandle)
-	kd->syncHandle = compAddTimeout (0, kconfigRcSync, (void *) d);
+    if (!kc->syncHandle)
+	kc->syncHandle = compAddTimeout (0, kconfigRcSync, 0);
 }
 
 static Bool
-kconfigStringToValue (CompDisplay     *d,
+kconfigStringToValue (CompObject      *object,
 		      QString	      str,
 		      CompOptionType  type,
 		      CompOptionValue *value)
@@ -276,10 +282,18 @@ kconfigStringToValue (CompDisplay     *d,
 	    return FALSE;
 	break;
     case CompOptionTypeKey:
-	stringToKeyAction (d, str.ascii (), &value->action);
+	while (object && object->type != COMP_OBJECT_TYPE_DISPLAY)
+	    object = object->parent;
+
+	stringToKeyAction (GET_CORE_DISPLAY (object), str.ascii (),
+			   &value->action);
 	break;
     case CompOptionTypeButton:
-	stringToButtonAction (d, str.ascii (), &value->action);
+	while (object && object->type != COMP_OBJECT_TYPE_DISPLAY)
+	    object = object->parent;
+
+	stringToButtonAction (GET_CORE_DISPLAY (object), str.ascii (),
+			      &value->action);
 	break;
     case CompOptionTypeEdge:
 	value->action.edgeMask = stringToEdgeMask (str.ascii ());
@@ -315,7 +329,7 @@ kconfigBoolToValue (bool	    b,
 }
 
 static Bool
-kconfigReadOptionValue (CompDisplay	*d,
+kconfigReadOptionValue (CompObject	*object,
 			KConfig		*config,
 			CompOption	*o,
 			CompOptionValue *value)
@@ -339,7 +353,8 @@ kconfigReadOptionValue (CompDisplay	*d,
     case CompOptionTypeButton:
     case CompOptionTypeEdge:
     case CompOptionTypeMatch:
-	if (!kconfigStringToValue (d, config->readEntry (o->name), o->type,
+	if (!kconfigStringToValue (object,
+				   config->readEntry (o->name), o->type,
 				   value))
 	    return FALSE;
 	break;
@@ -392,7 +407,7 @@ kconfigReadOptionValue (CompDisplay	*d,
 		{
 		    for (i = 0; i < n; i++)
 		    {
-			if (!kconfigStringToValue (d,
+			if (!kconfigStringToValue (object,
 						   list[i],
 						   value->list.type,
 						   &value->list.value[i]))
@@ -423,118 +438,83 @@ kconfigReadOptionValue (CompDisplay	*d,
 }
 
 static void
-kconfigGetDisplayOption (CompDisplay *d,
-			 CompOption  *o,
-			 const char  *plugin)
+kconfigGetOption (CompObject *object,
+		  CompOption *o,
+		  const char *plugin)
 {
-    QString       group (QString (plugin) + "_display");
+    QString	  group (QString (plugin) + "_" +
+			 kconfigObjectString (object));
     const QString name (o->name);
 
-    KCONFIG_DISPLAY (d);
+    KCONFIG_CORE (&core);
 
-    kd->config->setGroup (group);
+    kc->config->setGroup (group);
 
-    if (kd->config->hasKey (name))
+    if (kc->config->hasKey (name))
     {
 	CompOptionValue value;
 
-	if (kconfigReadOptionValue (d, kd->config, o, &value))
+	if (kconfigReadOptionValue (object, kc->config, o, &value))
 	{
-	    if (strcmp (plugin, "core") == 0)
-		(*d->setDisplayOption) (d, o->name, &value);
-	    else
-		(*d->setDisplayOptionForPlugin) (d, plugin, o->name, &value);
-
+	    (*core.setOptionForPlugin) (object, plugin, o->name, &value);
 	    compFiniOptionValue (&value, o->type);
 	}
     }
     else
     {
-	kconfigSetOption (d, o, plugin, "display");
+	kconfigSetOption (object, o, plugin);
     }
 }
 
-static void
-kconfigGetScreenOption (CompScreen *s,
-			CompOption *o,
-			const char *plugin,
-			const char *screen)
+static CompBool
+kconfigReloadObjectTree (CompObject *object,
+			 void       *closure);
+
+static CompBool
+kconfigReloadObjectsWithType (CompObjectType type,
+			      CompObject     *parent,
+			      void	     *closure)
 {
-    QString       group (QString (plugin) + "_" + QString (screen));
-    const QString name (o->name);
+    compObjectForEach (parent, type, kconfigReloadObjectTree, closure);
 
-    KCONFIG_DISPLAY (s->display);
+    return TRUE;
+}
 
-    kd->config->setGroup (group);
+static CompBool
+kconfigReloadObjectTree (CompObject *object,
+			 void       *closure)
+{
+    CompPlugin *p = (CompPlugin *) closure;
+    CompOption  *option;
+    int		nOption;
 
-    if (kd->config->hasKey (name))
-    {
-	CompOptionValue value;
+    option = (*p->vTable->getObjectOptions) (p, object, &nOption);
+    while (nOption--)
+	kconfigGetOption (object, option++, p->vTable->name);
 
-	if (kconfigReadOptionValue (s->display, kd->config, o, &value))
-	{
-	    if (strcmp (plugin, "core") == 0)
-		(*s->setScreenOption) (s, o->name, &value);
-	    else
-		(*s->setScreenOptionForPlugin) (s, plugin, o->name, &value);
+    compObjectForEachType (object, kconfigReloadObjectsWithType, closure);
 
-	    compFiniOptionValue (&value, o->type);
-	}
-    }
-    else
-    {
-	kconfigSetOption (s->display, o, plugin, screen);
-    }
+    return TRUE;
 }
 
 static Bool
 kconfigRcReload (void *closure)
 {
-    CompDisplay *d = (CompDisplay *) closure;
-    CompScreen  *s;
     CompPlugin  *p;
-    CompOption  *option;
-    int		nOption;
 
-    KCONFIG_DISPLAY (d);
+    KCONFIG_CORE (&core);
 
-    kd->config->reparseConfiguration ();
-
-    option = compGetDisplayOptions (d, &nOption);
-    while (nOption--)
-	kconfigGetDisplayOption (d, option++, "core");
+    kc->config->reparseConfiguration ();
 
     for (p = getPlugins (); p; p = p->next)
     {
-	if (!p->vTable->getDisplayOptions)
+	if (!p->vTable->getObjectOptions)
 	    continue;
 
-	option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-	while (nOption--)
-	    kconfigGetDisplayOption (d, option++, p->vTable->name);
+	kconfigReloadObjectTree (&core.base, (void *) p);
     }
 
-    for (s = d->screens; s; s = s->next)
-    {
-	QString screen ("screen" + QString::number (s->screenNum));
-
-	option = compGetScreenOptions (s, &nOption);
-	while (nOption--)
-	    kconfigGetScreenOption (s, option++, "core", screen.ascii ());
-
-	for (p = getPlugins (); p; p = p->next)
-	{
-	    if (!p->vTable->getScreenOptions)
-		continue;
-
-	    option = (*p->vTable->getScreenOptions) (p, s, &nOption);
-	    while (nOption--)
-		kconfigGetScreenOption (s, option++, p->vTable->name,
-					screen.ascii ());
-	}
-    }
-
-    kd->reloadHandle = 0;
+    kc->reloadHandle = 0;
 
     return FALSE;
 }
@@ -543,322 +523,164 @@ static void
 kconfigRcChanged (const char *name,
 		  void	     *closure)
 {
-    CompDisplay *d = (CompDisplay *) closure;
-
-    KCONFIG_DISPLAY (d);
-
     if (strcmp (name, COMPIZ_KCONFIG_RC) == 0)
     {
-	if (!kd->reloadHandle)
-	    kd->reloadHandle = compAddTimeout (0, kconfigRcReload, closure);
+	KCONFIG_CORE (&core);
+
+	if (!kc->reloadHandle)
+	    kc->reloadHandle = compAddTimeout (0, kconfigRcReload, closure);
     }
 }
 
-static Bool
-kconfigSetDisplayOption (CompDisplay     *d,
-			 const char	 *name,
-			 CompOptionValue *value)
+static CompBool
+kconfigSetOptionForPlugin (CompObject      *object,
+			   const char	   *plugin,
+			   const char	   *name,
+			   CompOptionValue *value)
 {
-    Bool status;
+    CompBool status;
 
-    KCONFIG_DISPLAY (d);
+    KCONFIG_CORE (&core);
 
-    UNWRAP (kd, d, setDisplayOption);
-    status = (*d->setDisplayOption) (d, name, value);
-    WRAP (kd, d, setDisplayOption, kconfigSetDisplayOption);
+    UNWRAP (kc, &core, setOptionForPlugin);
+    status = (*core.setOptionForPlugin) (object, plugin, name, value);
+    WRAP (kc, &core, setOptionForPlugin, kconfigSetOptionForPlugin);
 
-    if (status && !kd->reloadHandle)
-    {
-	CompOption *option;
-	int	   nOption;
-
-	option = compGetDisplayOptions (d, &nOption);
-	option = compFindOption (option, nOption, name, 0);
-	if (option)
-	    kconfigSetOption (d, option, "core", "display");
-    }
-
-    return status;
-}
-
-static Bool
-kconfigSetDisplayOptionForPlugin (CompDisplay     *d,
-				  const char	  *plugin,
-				  const char	  *name,
-				  CompOptionValue *value)
-{
-    Bool status;
-
-    KCONFIG_DISPLAY (d);
-
-    UNWRAP (kd, d, setDisplayOptionForPlugin);
-    status = (*d->setDisplayOptionForPlugin) (d, plugin, name, value);
-    WRAP (kd, d, setDisplayOptionForPlugin, kconfigSetDisplayOptionForPlugin);
-
-    if (status && !kd->reloadHandle)
+    if (status && !kc->reloadHandle)
     {
 	CompPlugin *p;
 
 	p = findActivePlugin (plugin);
-	if (p && p->vTable->getDisplayOptions)
+	if (p && p->vTable->getObjectOptions)
 	{
 	    CompOption *option;
 	    int	       nOption;
 
-	    option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
+	    option = (*p->vTable->getObjectOptions) (p, object, &nOption);
 	    option = compFindOption (option, nOption, name, 0);
 	    if (option)
-		kconfigSetOption (d, option, p->vTable->name, "display");
+		kconfigSetOption (object, option, p->vTable->name);
 	}
     }
 
     return status;
 }
 
-static Bool
-kconfigSetScreenOption (CompScreen      *s,
-			const char	*name,
-			CompOptionValue *value)
+static CompBool
+kconfigInitPluginForObject (CompPlugin *p,
+			    CompObject *o)
 {
-    Bool status;
+    CompBool status;
 
-    KCONFIG_SCREEN (s);
+    KCONFIG_CORE (&core);
 
-    UNWRAP (ks, s, setScreenOption);
-    status = (*s->setScreenOption) (s, name, value);
-    WRAP (ks, s, setScreenOption, kconfigSetScreenOption);
+    UNWRAP (kc, &core, initPluginForObject);
+    status = (*core.initPluginForObject) (p, o);
+    WRAP (kc, &core, initPluginForObject, kconfigInitPluginForObject);
 
-    if (status)
-    {
-	KCONFIG_DISPLAY (s->display);
-
-	if (!kd->reloadHandle)
-	{
-	    CompOption *option;
-	    int	       nOption;
-	    QString    screen ("screen");
-
-	    screen += QString::number (s->screenNum);
-
-	    option = compGetScreenOptions (s, &nOption);
-	    option = compFindOption (option, nOption, name, 0);
-	    if (option)
-		kconfigSetOption (s->display, option, "core", screen.ascii ());
-	}
-    }
-
-    return status;
-}
-
-static Bool
-kconfigSetScreenOptionForPlugin (CompScreen      *s,
-				 const char	 *plugin,
-				 const char	 *name,
-				 CompOptionValue *value)
-{
-    Bool status;
-
-    KCONFIG_SCREEN (s);
-
-    UNWRAP (ks, s, setScreenOptionForPlugin);
-    status = (*s->setScreenOptionForPlugin) (s, plugin, name, value);
-    WRAP (ks, s, setScreenOptionForPlugin, kconfigSetScreenOptionForPlugin);
-
-    if (status)
-    {
-	KCONFIG_DISPLAY (s->display);
-
-	if (!kd->reloadHandle)
-	{
-	    CompPlugin *p;
-
-	    p = findActivePlugin (plugin);
-	    if (p && p->vTable->getScreenOptions)
-	    {
-		CompOption *option;
-		int	   nOption;
-		QString    screen ("screen");
-
-		screen += QString::number (s->screenNum);
-
-		option = (*p->vTable->getScreenOptions) (p, s, &nOption);
-		option = compFindOption (option, nOption, name, 0);
-		if (option)
-		    kconfigSetOption (s->display, option, plugin,
-				      screen.ascii ());
-	    }
-	}
-    }
-
-    return status;
-}
-
-static Bool
-kconfigInitPluginForDisplay (CompPlugin  *p,
-			     CompDisplay *d)
-{
-    Bool status;
-
-    KCONFIG_DISPLAY (d);
-
-    UNWRAP (kd, d, initPluginForDisplay);
-    status = (*d->initPluginForDisplay) (p, d);
-    WRAP (kd, d, initPluginForDisplay, kconfigInitPluginForDisplay);
-
-    if (status && p->vTable->getDisplayOptions)
+    if (status && p->vTable->getObjectOptions)
     {
 	CompOption *option;
 	int	   nOption;
 
-	option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
+	option = (*p->vTable->getObjectOptions) (p, o, &nOption);
 	while (nOption--)
-	    kconfigGetDisplayOption (d, option++, p->vTable->name);
+	    kconfigGetOption (o, option++, p->vTable->name);
     }
 
     return status;
 }
 
 static Bool
-kconfigInitPluginForScreen (CompPlugin *p,
-			    CompScreen *s)
+kconfigInitCore (CompPlugin *p,
+		 CompCore   *c)
 {
-    Bool status;
+    KconfigCore *kc;
+    QString	dir;
 
-    KCONFIG_SCREEN (s);
-
-    UNWRAP (ks, s, initPluginForScreen);
-    status = (*s->initPluginForScreen) (p, s);
-    WRAP (ks, s, initPluginForScreen, kconfigInitPluginForScreen);
-
-    if (status && p->vTable->getScreenOptions)
-    {
-	CompOption *option;
-	int	   nOption;
-	QString    screen ("screen");
-
-	screen += QString::number (s->screenNum);
-
-	option = (*p->vTable->getScreenOptions) (p, s, &nOption);
-	while (nOption--)
-	    kconfigGetScreenOption (s, option++, p->vTable->name,
-				    screen.ascii ());
-    }
-
-    return status;
-}
-
-static Bool
-kconfigInitDisplay (CompPlugin  *p,
-		    CompDisplay *d)
-{
-    KconfigDisplay *kd;
-    QString	   dir;
-
-    kd = new KconfigDisplay;
-    if (!kd)
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
 	return FALSE;
 
-    kd->config = new KConfig (COMPIZ_KCONFIG_RC);
-    if (!kd->config)
+    kc = new KconfigCore;
+    if (!kc)
+	return FALSE;
+
+    kc->config = new KConfig (COMPIZ_KCONFIG_RC);
+    if (!kc->config)
     {
-	delete kd;
+	delete kc;
 	return FALSE;
     }
 
-    kd->screenPrivateIndex = allocateScreenPrivateIndex (d);
-    if (kd->screenPrivateIndex < 0)
-    {
-	delete kd->config;
-	delete kd;
-	return FALSE;
-    }
-
-    kd->reloadHandle = compAddTimeout (0, kconfigRcReload, (void *) d);
-    kd->syncHandle   = 0;
-    kd->fileWatch    = 0;
+    kc->reloadHandle = compAddTimeout (0, kconfigRcReload, 0);
+    kc->syncHandle   = 0;
+    kc->fileWatch    = 0;
 
     dir = KGlobal::dirs ()->saveLocation ("config", QString::null, false);
 
     if (QFile::exists (dir))
     {
-	kd->fileWatch = addFileWatch (d, dir.ascii (), ~0, kconfigRcChanged,
-				      (void *) d);
+	kc->fileWatch = addFileWatch (dir.ascii (), ~0, kconfigRcChanged, 0);
     }
     else
     {
-	compLogMessage (d, "kconfig", CompLogLevelWarn, "Bad access \"%s\"",
+	compLogMessage (0, "kconfig", CompLogLevelWarn, "Bad access \"%s\"",
 			dir.ascii ());
     }
 
-    WRAP (kd, d, initPluginForDisplay, kconfigInitPluginForDisplay);
-    WRAP (kd, d, setDisplayOption, kconfigSetDisplayOption);
-    WRAP (kd, d, setDisplayOptionForPlugin, kconfigSetDisplayOptionForPlugin);
+    WRAP (kc, c, initPluginForObject, kconfigInitPluginForObject);
+    WRAP (kc, c, setOptionForPlugin, kconfigSetOptionForPlugin);
 
-    d->privates[displayPrivateIndex].ptr = kd;
+    c->base.privates[corePrivateIndex].ptr = kc;
 
     return TRUE;
 }
 
 static void
-kconfigFiniDisplay (CompPlugin  *p,
-		    CompDisplay *d)
+kconfigFiniCore (CompPlugin *p,
+		 CompCore   *c)
 {
-    KCONFIG_DISPLAY (d);
+    KCONFIG_CORE (c);
 
-    UNWRAP (kd, d, initPluginForDisplay);
-    UNWRAP (kd, d, setDisplayOption);
-    UNWRAP (kd, d, setDisplayOptionForPlugin);
+    UNWRAP (kc, c, initPluginForObject);
+    UNWRAP (kc, c, setOptionForPlugin);
 
-    if (kd->reloadHandle)
-	compRemoveTimeout (kd->reloadHandle);
+    if (kc->reloadHandle)
+	compRemoveTimeout (kc->reloadHandle);
 
-    if (kd->syncHandle)
+    if (kc->syncHandle)
     {
-	compRemoveTimeout (kd->syncHandle);
-	kconfigRcSync (d);
+	compRemoveTimeout (kc->syncHandle);
+	kconfigRcSync (0);
     }
 
-    if (kd->fileWatch)
-	removeFileWatch (d, kd->fileWatch);
+    if (kc->fileWatch)
+	removeFileWatch (kc->fileWatch);
 
-    freeScreenPrivateIndex (d, kd->screenPrivateIndex);
-
-    delete kd->config;
-    delete kd;
+    delete kc->config;
+    delete kc;
 }
 
-static Bool
-kconfigInitScreen (CompPlugin *p,
-		   CompScreen *s)
+static CompBool
+kconfigInitObject (CompPlugin *p,
+		   CompObject *o)
 {
-    KconfigScreen *ks;
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) kconfigInitCore
+    };
 
-    KCONFIG_DISPLAY (s->display);
-
-    ks = new KconfigScreen;
-    if (!ks)
-	return FALSE;
-
-    WRAP (ks, s, initPluginForScreen, kconfigInitPluginForScreen);
-    WRAP (ks, s, setScreenOption, kconfigSetScreenOption);
-    WRAP (ks, s, setScreenOptionForPlugin, kconfigSetScreenOptionForPlugin);
-
-    s->privates[kd->screenPrivateIndex].ptr = ks;
-
-    return TRUE;
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
 }
 
 static void
-kconfigFiniScreen (CompPlugin *p,
-		   CompScreen *s)
+kconfigFiniObject (CompPlugin *p,
+		   CompObject *o)
 {
-    KCONFIG_SCREEN (s);
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) kconfigFiniCore
+    };
 
-    UNWRAP (ks, s, initPluginForScreen);
-    UNWRAP (ks, s, setScreenOption);
-    UNWRAP (ks, s, setScreenOptionForPlugin);
-
-    delete ks;
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
 }
 
 static Bool
@@ -868,8 +690,8 @@ kconfigInit (CompPlugin *p)
 					 0, 0, 0, 0))
 	return FALSE;
 
-    displayPrivateIndex = allocateDisplayPrivateIndex ();
-    if (displayPrivateIndex < 0)
+    corePrivateIndex = allocateCorePrivateIndex ();
+    if (corePrivateIndex < 0)
     {
 	compFiniMetadata (&kconfigMetadata);
 	return FALSE;
@@ -878,7 +700,7 @@ kconfigInit (CompPlugin *p)
     kInstance = new KInstance ("compiz-kconfig");
     if (!kInstance)
     {
-	freeDisplayPrivateIndex (displayPrivateIndex);
+	freeCorePrivateIndex (corePrivateIndex);
 	compFiniMetadata (&kconfigMetadata);
 	return FALSE;
     }
@@ -893,17 +715,10 @@ kconfigFini (CompPlugin *p)
 {
     delete kInstance;
 
-    freeDisplayPrivateIndex (displayPrivateIndex);
+    freeCorePrivateIndex (corePrivateIndex);
     compFiniMetadata (&kconfigMetadata);
 }
 
-static int
-kconfigGetVersion (CompPlugin *plugin,
-		 int	    version)
-{
-    return ABIVERSION;
-}
-
 static CompMetadata *
 kconfigGetMetadata (CompPlugin *plugin)
 {
@@ -912,24 +727,17 @@ kconfigGetMetadata (CompPlugin *plugin)
 
 CompPluginVTable kconfigVTable = {
     "kconfig",
-    kconfigGetVersion,
     kconfigGetMetadata,
     kconfigInit,
     kconfigFini,
-    kconfigInitDisplay,
-    kconfigFiniDisplay,
-    kconfigInitScreen,
-    kconfigFiniScreen,
-    0, /* InitWindow */
-    0, /* FiniWindow */
-    0, /* GetDisplayOptions */
-    0, /* SetDisplayOption */
-    0, /* GetScreenOptions */
-    0  /* SetScreenOption */
+    kconfigInitObject,
+    kconfigFiniObject,
+    0, /* GetObjectOptions */
+    0  /* SetObjectOption */
 };
 
 CompPluginVTable *
-getCompPluginInfo (void)
+getCompPluginInfo20070830 (void)
 {
     return &kconfigVTable;
 }
