@@ -29,7 +29,7 @@
 
 #include <X11/cursorfont.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 
 static CompMetadata moveMetadata;
 
@@ -92,14 +92,14 @@ typedef struct _MoveScreen {
     int	snapBackY;
 } MoveScreen;
 
-#define GET_MOVE_DISPLAY(d)				     \
-    ((MoveDisplay *) (d)->privates[displayPrivateIndex].ptr)
+#define GET_MOVE_DISPLAY(d)					  \
+    ((MoveDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 
 #define MOVE_DISPLAY(d)		           \
     MoveDisplay *md = GET_MOVE_DISPLAY (d)
 
-#define GET_MOVE_SCREEN(s, md)				         \
-    ((MoveScreen *) (s)->privates[(md)->screenPrivateIndex].ptr)
+#define GET_MOVE_SCREEN(s, md)					      \
+    ((MoveScreen *) (s)->base.privates[(md)->screenPrivateIndex].ptr)
 
 #define MOVE_SCREEN(s)						        \
     MoveScreen *ms = GET_MOVE_SCREEN (s, GET_MOVE_DISPLAY (s->display))
@@ -446,7 +446,7 @@ moveHandleMotionEvent (CompScreen *s,
 	    {
 		if (w->state & CompWindowStateMaximizedVertMask)
 		{
-		    if ((yRoot - workArea.y) - ms->snapOffY >= SNAP_OFF)
+		    if (abs ((yRoot - workArea.y) - ms->snapOffY) >= SNAP_OFF)
 		    {
 			if (!otherScreenGrabExist (s, "move", 0))
 			{
@@ -472,7 +472,7 @@ moveHandleMotionEvent (CompScreen *s,
 		}
 		else if (ms->origState & CompWindowStateMaximizedVertMask)
 		{
-		    if ((yRoot - workArea.y) - ms->snapBackY < SNAP_BACK)
+		    if (abs ((yRoot - workArea.y) - ms->snapBackY) < SNAP_BACK)
 		    {
 			if (!otherScreenGrabExist (s, "move", 0))
 			{
@@ -806,6 +806,9 @@ moveInitDisplay (CompPlugin  *p,
     MoveDisplay *md;
     int	        i;
 
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
+	return FALSE;
+
     md = malloc (sizeof (MoveDisplay));
     if (!md)
 	return FALSE;
@@ -841,7 +844,7 @@ moveInitDisplay (CompPlugin  *p,
 
     WRAP (md, d, handleEvent, moveHandleEvent);
 
-    d->privates[displayPrivateIndex].ptr = md;
+    d->base.privates[displayPrivateIndex].ptr = md;
 
     return TRUE;
 }
@@ -879,7 +882,7 @@ moveInitScreen (CompPlugin *p,
 
     WRAP (ms, s, paintWindow, movePaintWindow);
 
-    s->privates[md->screenPrivateIndex].ptr = ms;
+    s->base.privates[md->screenPrivateIndex].ptr = ms;
 
     return TRUE;
 }
@@ -898,6 +901,61 @@ moveFiniScreen (CompPlugin *p,
     free (ms);
 }
 
+static CompBool
+moveInitObject (CompPlugin *p,
+		CompObject *o)
+{
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) 0, /* InitCore */
+	(InitPluginObjectProc) moveInitDisplay,
+	(InitPluginObjectProc) moveInitScreen
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+moveFiniObject (CompPlugin *p,
+		CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) 0, /* FiniCore */
+	(FiniPluginObjectProc) moveFiniDisplay,
+	(FiniPluginObjectProc) moveFiniScreen
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
+}
+
+static CompOption *
+moveGetObjectOptions (CompPlugin *plugin,
+		      CompObject *object,
+		      int	 *count)
+{
+    static GetPluginObjectOptionsProc dispTab[] = {
+	(GetPluginObjectOptionsProc) 0, /* GetCoreOptions */
+	(GetPluginObjectOptionsProc) moveGetDisplayOptions
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
+		     (void *) (*count = 0), (plugin, object, count));
+}
+
+static CompBool
+moveSetObjectOption (CompPlugin      *plugin,
+		     CompObject      *object,
+		     const char      *name,
+		     CompOptionValue *value)
+{
+    static SetPluginObjectOptionProc dispTab[] = {
+	(SetPluginObjectOptionProc) 0, /* SetCoreOption */
+	(SetPluginObjectOptionProc) moveSetDisplayOption
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab), FALSE,
+		     (plugin, object, name, value));
+}
+
 static Bool
 moveInit (CompPlugin *p)
 {
@@ -927,13 +985,6 @@ moveFini (CompPlugin *p)
     compFiniMetadata (&moveMetadata);
 }
 
-static int
-moveGetVersion (CompPlugin *plugin,
-		int	   version)
-{
-    return ABIVERSION;
-}
-
 static CompMetadata *
 moveGetMetadata (CompPlugin *plugin)
 {
@@ -942,24 +993,17 @@ moveGetMetadata (CompPlugin *plugin)
 
 CompPluginVTable moveVTable = {
     "move",
-    moveGetVersion,
     moveGetMetadata,
     moveInit,
     moveFini,
-    moveInitDisplay,
-    moveFiniDisplay,
-    moveInitScreen,
-    moveFiniScreen,
-    0, /* InitWindow */
-    0, /* FiniWindow */
-    moveGetDisplayOptions,
-    moveSetDisplayOption,
-    0, /* GetScreenOptions */
-    0  /* SetScreenOption */
+    moveInitObject,
+    moveFiniObject,
+    moveGetObjectOptions,
+    moveSetObjectOption
 };
 
 CompPluginVTable *
-getCompPluginInfo (void)
+getCompPluginInfo20070830 (void)
 {
     return &moveVTable;
 }
