@@ -32,7 +32,7 @@
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
 
-#include <cube.h>
+#include <compiz-cube.h>
 
 static int cubeDisplayPrivateIndex;
 
@@ -44,43 +44,47 @@ static int displayPrivateIndex;
 
 #define ROTATE_DISPLAY_OPTION_INITIATE_BUTTON	  0
 #define ROTATE_DISPLAY_OPTION_LEFT_KEY		  1
-#define ROTATE_DISPLAY_OPTION_RIGHT_KEY		  2
-#define ROTATE_DISPLAY_OPTION_LEFT_WINDOW_KEY	  3
-#define ROTATE_DISPLAY_OPTION_RIGHT_WINDOW_KEY	  4
-#define ROTATE_DISPLAY_OPTION_EDGEFLIP_POINTER	  5
-#define ROTATE_DISPLAY_OPTION_EDGEFLIP_WINDOW	  6
-#define ROTATE_DISPLAY_OPTION_EDGEFLIP_DND	  7
-#define ROTATE_DISPLAY_OPTION_FLIPTIME		  8
-#define ROTATE_DISPLAY_OPTION_TO_1_KEY		  9
-#define ROTATE_DISPLAY_OPTION_TO_2_KEY		  10
-#define ROTATE_DISPLAY_OPTION_TO_3_KEY		  11
-#define ROTATE_DISPLAY_OPTION_TO_4_KEY		  12
-#define ROTATE_DISPLAY_OPTION_TO_5_KEY		  13
-#define ROTATE_DISPLAY_OPTION_TO_6_KEY		  14
-#define ROTATE_DISPLAY_OPTION_TO_7_KEY		  15
-#define ROTATE_DISPLAY_OPTION_TO_8_KEY		  16
-#define ROTATE_DISPLAY_OPTION_TO_9_KEY		  17
-#define ROTATE_DISPLAY_OPTION_TO_10_KEY		  18
-#define ROTATE_DISPLAY_OPTION_TO_11_KEY		  19
-#define ROTATE_DISPLAY_OPTION_TO_12_KEY		  20
-#define ROTATE_DISPLAY_OPTION_TO_1_WINDOW_KEY	  21
-#define ROTATE_DISPLAY_OPTION_TO_2_WINDOW_KEY	  22
-#define ROTATE_DISPLAY_OPTION_TO_3_WINDOW_KEY	  23
-#define ROTATE_DISPLAY_OPTION_TO_4_WINDOW_KEY	  24
-#define ROTATE_DISPLAY_OPTION_TO_5_WINDOW_KEY	  25
-#define ROTATE_DISPLAY_OPTION_TO_6_WINDOW_KEY	  26
-#define ROTATE_DISPLAY_OPTION_TO_7_WINDOW_KEY	  27
-#define ROTATE_DISPLAY_OPTION_TO_8_WINDOW_KEY	  28
-#define ROTATE_DISPLAY_OPTION_TO_9_WINDOW_KEY	  29
-#define ROTATE_DISPLAY_OPTION_TO_10_WINDOW_KEY	  30
-#define ROTATE_DISPLAY_OPTION_TO_11_WINDOW_KEY	  31
-#define ROTATE_DISPLAY_OPTION_TO_12_WINDOW_KEY	  32
-#define ROTATE_DISPLAY_OPTION_TO_KEY		  33
-#define ROTATE_DISPLAY_OPTION_WINDOW_KEY	  34
-#define ROTATE_DISPLAY_OPTION_FLIP_LEFT_EDGE	  35
-#define ROTATE_DISPLAY_OPTION_FLIP_RIGHT_EDGE	  36
-#define ROTATE_DISPLAY_OPTION_RAISE_ON_ROTATE	  37
-#define ROTATE_DISPLAY_OPTION_NUM		  38
+#define ROTATE_DISPLAY_OPTION_LEFT_BUTTON	  2
+#define ROTATE_DISPLAY_OPTION_RIGHT_KEY		  3
+#define ROTATE_DISPLAY_OPTION_RIGHT_BUTTON	  4
+#define ROTATE_DISPLAY_OPTION_LEFT_WINDOW_KEY	  5
+#define ROTATE_DISPLAY_OPTION_LEFT_WINDOW_BUTTON  6
+#define ROTATE_DISPLAY_OPTION_RIGHT_WINDOW_KEY	  7
+#define ROTATE_DISPLAY_OPTION_RIGHT_WINDOW_BUTTON 8
+#define ROTATE_DISPLAY_OPTION_EDGEFLIP_POINTER	  9
+#define ROTATE_DISPLAY_OPTION_EDGEFLIP_WINDOW	  10
+#define ROTATE_DISPLAY_OPTION_EDGEFLIP_DND	  11
+#define ROTATE_DISPLAY_OPTION_FLIPTIME		  12
+#define ROTATE_DISPLAY_OPTION_TO_1_KEY		  13
+#define ROTATE_DISPLAY_OPTION_TO_2_KEY		  14
+#define ROTATE_DISPLAY_OPTION_TO_3_KEY		  15
+#define ROTATE_DISPLAY_OPTION_TO_4_KEY		  16
+#define ROTATE_DISPLAY_OPTION_TO_5_KEY		  17
+#define ROTATE_DISPLAY_OPTION_TO_6_KEY		  18
+#define ROTATE_DISPLAY_OPTION_TO_7_KEY		  19
+#define ROTATE_DISPLAY_OPTION_TO_8_KEY		  20
+#define ROTATE_DISPLAY_OPTION_TO_9_KEY		  21
+#define ROTATE_DISPLAY_OPTION_TO_10_KEY		  22
+#define ROTATE_DISPLAY_OPTION_TO_11_KEY		  23
+#define ROTATE_DISPLAY_OPTION_TO_12_KEY		  24
+#define ROTATE_DISPLAY_OPTION_TO_1_WINDOW_KEY	  25
+#define ROTATE_DISPLAY_OPTION_TO_2_WINDOW_KEY	  26
+#define ROTATE_DISPLAY_OPTION_TO_3_WINDOW_KEY	  27
+#define ROTATE_DISPLAY_OPTION_TO_4_WINDOW_KEY	  28
+#define ROTATE_DISPLAY_OPTION_TO_5_WINDOW_KEY	  29
+#define ROTATE_DISPLAY_OPTION_TO_6_WINDOW_KEY	  30
+#define ROTATE_DISPLAY_OPTION_TO_7_WINDOW_KEY	  31
+#define ROTATE_DISPLAY_OPTION_TO_8_WINDOW_KEY	  32
+#define ROTATE_DISPLAY_OPTION_TO_9_WINDOW_KEY	  33
+#define ROTATE_DISPLAY_OPTION_TO_10_WINDOW_KEY	  34
+#define ROTATE_DISPLAY_OPTION_TO_11_WINDOW_KEY	  35
+#define ROTATE_DISPLAY_OPTION_TO_12_WINDOW_KEY	  36
+#define ROTATE_DISPLAY_OPTION_TO_KEY		  37
+#define ROTATE_DISPLAY_OPTION_WINDOW_KEY	  38
+#define ROTATE_DISPLAY_OPTION_FLIP_LEFT_EDGE	  39
+#define ROTATE_DISPLAY_OPTION_FLIP_RIGHT_EDGE	  40
+#define ROTATE_DISPLAY_OPTION_RAISE_ON_ROTATE	  41
+#define ROTATE_DISPLAY_OPTION_NUM		  42
 
 typedef struct _RotateDisplay {
     int		    screenPrivateIndex;
@@ -93,10 +97,11 @@ typedef struct _RotateDisplay {
 #define ROTATE_SCREEN_OPTION_POINTER_SENSITIVITY 1
 #define ROTATE_SCREEN_OPTION_ACCELERATION        2
 #define ROTATE_SCREEN_OPTION_SNAP_TOP		 3
-#define ROTATE_SCREEN_OPTION_SPEED		 4
-#define ROTATE_SCREEN_OPTION_TIMESTEP		 5
-#define ROTATE_SCREEN_OPTION_ZOOM		 6
-#define ROTATE_SCREEN_OPTION_NUM		 7
+#define ROTATE_SCREEN_OPTION_SNAP_BOTTOM	 4
+#define ROTATE_SCREEN_OPTION_SPEED		 5
+#define ROTATE_SCREEN_OPTION_TIMESTEP		 6
+#define ROTATE_SCREEN_OPTION_ZOOM		 7
+#define ROTATE_SCREEN_OPTION_NUM		 8
 
 typedef struct _RotateScreen {
     PreparePaintScreenProc	 preparePaintScreen;
@@ -113,6 +118,7 @@ typedef struct _RotateScreen {
     float pointerSensitivity;
 
     Bool snapTop;
+    Bool snapBottom;
 
     int grabIndex;
 
@@ -139,14 +145,14 @@ typedef struct _RotateScreen {
     GLfloat zoomVelocity;
 } RotateScreen;
 
-#define GET_ROTATE_DISPLAY(d)				       \
-    ((RotateDisplay *) (d)->privates[displayPrivateIndex].ptr)
+#define GET_ROTATE_DISPLAY(d)					    \
+    ((RotateDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 
 #define ROTATE_DISPLAY(d)		       \
     RotateDisplay *rd = GET_ROTATE_DISPLAY (d)
 
-#define GET_ROTATE_SCREEN(s, rd)				   \
-    ((RotateScreen *) (s)->privates[(rd)->screenPrivateIndex].ptr)
+#define GET_ROTATE_SCREEN(s, rd)					\
+    ((RotateScreen *) (s)->base.privates[(rd)->screenPrivateIndex].ptr)
 
 #define ROTATE_SCREEN(s)						      \
     RotateScreen *rs = GET_ROTATE_SCREEN (s, GET_ROTATE_DISPLAY (s->display))
@@ -197,7 +203,8 @@ rotateSetScreenOption (CompPlugin      *plugin,
 
 static int
 adjustVelocity (RotateScreen *rs,
-		int	     size)
+		int	     size,
+		int	     invert)
 {
     float xrot, yrot, adjust, amount;
 
@@ -226,10 +233,17 @@ adjustVelocity (RotateScreen *rs,
 
     rs->xVelocity = (amount * rs->xVelocity + adjust) / (amount + 2.0f);
 
-    if (rs->snapTop && rs->yrot > 50.0f)
-	yrot = -(90.f - rs->yrot);
-    else
-	yrot = rs->yrot;
+    yrot = rs->yrot;
+    /* Only snap if more than 2 viewports */
+    if (size > 2)
+    {
+	if (rs->yrot > 50.0f && ((rs->snapTop && invert == 1) ||
+				 (rs->snapBottom && invert != 1)))
+	    yrot -= 90.f;
+	else if (rs->yrot < -50.0f && ((rs->snapTop && invert != 1) ||
+				       (rs->snapBottom && invert == 1)))
+	    yrot += 90.f;
+    }
 
     adjust = -yrot * 0.05f * rs->opt[ROTATE_SCREEN_OPTION_ACCELERATION].value.f;
     amount = fabs (rs->yrot);
@@ -332,7 +346,7 @@ rotatePreparePaintScreen (CompScreen *s,
 		if (fabs (rs->yVelocity) < 0.01f)
 		    rs->yVelocity = 0.0f;
 	    }
-	    else if (adjustVelocity (rs, s->hsize))
+	    else if (adjustVelocity (rs, s->hsize, cs->invert))
 	    {
 		rs->xVelocity = 0.0f;
 		rs->yVelocity = 0.0f;
@@ -387,7 +401,7 @@ rotatePreparePaintScreen (CompScreen *s,
 
 			/* only focus default window if switcher isn't active */
 			if (i == s->maxGrab)
-			    focusDefaultWindow (s->display);
+			    focusDefaultWindow (s);
 		    }
 
 		    rs->moveWindow = 0;
@@ -499,8 +513,11 @@ rotateDonePaintScreen (CompScreen *s)
 
     if (rs->grabIndex || rs->moving)
     {
-	if ((!rs->grabbed && !rs->snapTop) || rs->xVelocity || rs->yVelocity)
+	if ((!rs->grabbed && !rs->snapTop && !rs->snapBottom) ||
+	    rs->xVelocity || rs->yVelocity)
+	{
 	    damageScreen (s);
+	}
     }
 
     if (rs->zoomTranslate > 0.0f &&
@@ -555,20 +572,30 @@ rotatePaintOutput (CompScreen		   *s,
 		   unsigned int		   mask)
 {
     Bool status;
-    ScreenPaintAttrib sA = *sAttrib;
 
     ROTATE_SCREEN (s);
 
     if (rs->grabIndex || rs->moving || rs->zoomTranslate != 0.0f)
     {
-	sA.zCamera -= rs->zoomTranslate;
+	CompTransform sTransform = *transform;
+
+	matrixTranslate (&sTransform, 0.0f, 0.0f, -rs->zoomTranslate);
+
 	mask &= ~PAINT_SCREEN_REGION_MASK;
 	mask |= PAINT_SCREEN_TRANSFORMED_MASK;
-    }
 
-    UNWRAP (rs, s, paintOutput);
-    status = (*s->paintOutput) (s, &sA, transform, region, output, mask);
-    WRAP (rs, s, paintOutput, rotatePaintOutput);
+	UNWRAP (rs, s, paintOutput);
+	status = (*s->paintOutput) (s, sAttrib, &sTransform, region,
+				    output, mask);
+	WRAP (rs, s, paintOutput, rotatePaintOutput);
+    }
+    else
+    {
+	UNWRAP (rs, s, paintOutput);
+	status = (*s->paintOutput) (s, sAttrib, transform, region,
+				    output, mask);
+	WRAP (rs, s, paintOutput, rotatePaintOutput);
+    }
 
     return status;
 }
@@ -637,6 +664,7 @@ rotateInitiate (CompDisplay     *d,
 
 	    rs->grabbed = TRUE;
 	    rs->snapTop = rs->opt[ROTATE_SCREEN_OPTION_SNAP_TOP].value.b;
+	    rs->snapBottom = rs->opt[ROTATE_SCREEN_OPTION_SNAP_BOTTOM].value.b;
 
 	    if (state & CompActionStateInitButton)
 		action->state |= CompActionStateTermButton;
@@ -671,7 +699,10 @@ rotateTerminate (CompDisplay     *d,
 	if (rs->grabIndex)
 	{
 	    if (!xid)
+	    {
 		rs->snapTop = FALSE;
+		rs->snapBottom = FALSE;
+	    }
 
 	    rs->grabbed = FALSE;
 	    damageScreen (s);
@@ -1670,9 +1701,13 @@ rotateSetDisplayOption (CompPlugin      *plugin,
 static const CompMetadataOptionInfo rotateDisplayOptionInfo[] = {
     { "initiate_button", "button", 0, rotateInitiate, rotateTerminate },
     { "rotate_left_key", "key", 0, rotateLeft, 0 },
+    { "rotate_left_button", "button", 0, rotateLeft, 0 },
     { "rotate_right_key", "key", 0, rotateRight, 0 },
+    { "rotate_right_button", "button", 0, rotateRight, 0 },
     { "rotate_left_window_key", "key", 0, rotateLeftWithWindow, 0 },
+    { "rotate_left_window_button", "button", 0, rotateLeftWithWindow, 0 },
     { "rotate_right_window_key", "key", 0, rotateRightWithWindow, 0 },
+    { "rotate_right_window_button", "button", 0, rotateRightWithWindow, 0 },
     { "edge_flip_pointer", "bool", 0, 0, 0 },
     { "edge_flip_window", "bool", 0, 0, 0 },
     { "edge_flip_dnd", "bool", 0, 0, 0 },
@@ -1715,24 +1750,12 @@ rotateInitDisplay (CompPlugin  *p,
 		   CompDisplay *d)
 {
     RotateDisplay *rd;
-    CompPlugin	  *cube = findActivePlugin ("cube");
-    CompOption	  *option;
-    int		  nOption;
 
-    if (!cube || !cube->vTable->getDisplayOptions)
+    if (!checkPluginABI ("core", CORE_ABIVERSION) ||
+	!checkPluginABI ("cube", CUBE_ABIVERSION))
 	return FALSE;
 
-    option = (*cube->vTable->getDisplayOptions) (cube, d, &nOption);
-
-    if (getIntOptionNamed (option, nOption, "abi", 0) != CUBE_ABIVERSION)
-    {
-	compLogMessage (d, "rotate", CompLogLevelError,
-			"cube ABI version mismatch");
-	return FALSE;
-    }
-
-    cubeDisplayPrivateIndex = getIntOptionNamed (option, nOption, "index", -1);
-    if (cubeDisplayPrivateIndex < 0)
+    if (!getPluginDisplayIndex (d, "cube", &cubeDisplayPrivateIndex))
 	return FALSE;
 
     rd = malloc (sizeof (RotateDisplay));
@@ -1759,7 +1782,7 @@ rotateInitDisplay (CompPlugin  *p,
 
     WRAP (rd, d, handleEvent, rotateHandleEvent);
 
-    d->privates[displayPrivateIndex].ptr = rd;
+    d->base.privates[displayPrivateIndex].ptr = rd;
 
     return TRUE;
 }
@@ -1784,6 +1807,7 @@ static const CompMetadataOptionInfo rotateScreenOptionInfo[] = {
     { "sensitivity", "float", 0, 0, 0 },
     { "acceleration", "float", "<min>1.0</min>", 0, 0 },
     { "snap_top", "bool", 0, 0, 0 },
+    { "snap_bottom", "bool", 0, 0, 0 },
     { "speed", "float", "<min>0.1</min>", 0, 0 },
     { "timestep", "float", "<min>0.1</min>", 0, 0 },
     { "zoom", "float", 0, 0, 0 }
@@ -1829,8 +1853,9 @@ rotateInitScreen (CompPlugin *p,
     rs->savedPointer.x = 0;
     rs->savedPointer.y = 0;
 
-    rs->grabbed = FALSE;
-    rs->snapTop = FALSE;
+    rs->grabbed	   = FALSE;
+    rs->snapTop	   = FALSE;
+    rs->snapBottom = FALSE;
 
     rs->slow       = FALSE;
     rs->grabMask   = FALSE;
@@ -1854,7 +1879,7 @@ rotateInitScreen (CompPlugin *p,
 
     WRAP (rs, cs, getRotation, rotateGetRotation);
 
-    s->privates[rd->screenPrivateIndex].ptr = rs;
+    s->base.privates[rd->screenPrivateIndex].ptr = rs;
 
     return TRUE;
 }
@@ -1879,6 +1904,62 @@ rotateFiniScreen (CompPlugin *p,
 
     free (rs);
 }
+static CompBool
+rotateInitObject (CompPlugin *p,
+		  CompObject *o)
+{
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) 0, /* InitCore */
+	(InitPluginObjectProc) rotateInitDisplay,
+	(InitPluginObjectProc) rotateInitScreen
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+rotateFiniObject (CompPlugin *p,
+		  CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) 0, /* FiniCore */
+	(FiniPluginObjectProc) rotateFiniDisplay,
+	(FiniPluginObjectProc) rotateFiniScreen
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
+}
+
+static CompOption *
+rotateGetObjectOptions (CompPlugin *plugin,
+			CompObject *object,
+			int	   *count)
+{
+    static GetPluginObjectOptionsProc dispTab[] = {
+	(GetPluginObjectOptionsProc) 0, /* GetCoreOptions */
+	(GetPluginObjectOptionsProc) rotateGetDisplayOptions,
+	(GetPluginObjectOptionsProc) rotateGetScreenOptions
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
+		     (void *) (*count = 0), (plugin, object, count));
+}
+
+static CompBool
+rotateSetObjectOption (CompPlugin      *plugin,
+		       CompObject      *object,
+		       const char      *name,
+		       CompOptionValue *value)
+{
+    static SetPluginObjectOptionProc dispTab[] = {
+	(SetPluginObjectOptionProc) 0, /* SetCoreOption */
+	(SetPluginObjectOptionProc) rotateSetDisplayOption,
+	(SetPluginObjectOptionProc) rotateSetScreenOption
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab), FALSE,
+		     (plugin, object, name, value));
+}
 
 static Bool
 rotateInit (CompPlugin *p)
@@ -1910,13 +1991,6 @@ rotateFini (CompPlugin *p)
     compFiniMetadata (&rotateMetadata);
 }
 
-static int
-rotateGetVersion (CompPlugin *plugin,
-		  int	     version)
-{
-    return ABIVERSION;
-}
-
 static CompMetadata *
 rotateGetMetadata (CompPlugin *plugin)
 {
@@ -1925,24 +1999,17 @@ rotateGetMetadata (CompPlugin *plugin)
 
 CompPluginVTable rotateVTable = {
     "rotate",
-    rotateGetVersion,
     rotateGetMetadata,
     rotateInit,
     rotateFini,
-    rotateInitDisplay,
-    rotateFiniDisplay,
-    rotateInitScreen,
-    rotateFiniScreen,
-    0, /* InitWindow */
-    0, /* FiniWindow */
-    rotateGetDisplayOptions,
-    rotateSetDisplayOption,
-    rotateGetScreenOptions,
-    rotateSetScreenOption
+    rotateInitObject,
+    rotateFiniObject,
+    rotateGetObjectOptions,
+    rotateSetObjectOption
 };
 
 CompPluginVTable *
-getCompPluginInfo (void)
+getCompPluginInfo20070830 (void)
 {
     return &rotateVTable;
 }
