@@ -32,7 +32,7 @@
 #include <X11/Xatom.h>
 #include <X11/cursorfont.h>
 
-#include <scale.h>
+#include <compiz-scale.h>
 
 #define EDGE_STATE (CompActionStateInitEdge)
 
@@ -48,7 +48,7 @@ static int scaleDisplayPrivateIndex;
 #define NUM_OPTIONS(s) (sizeof ((s)->opt) / sizeof (CompOption))
 
 static CompOption *
-scaleGetScreenOptions (CompPlugin  *plugin,
+scaleGetScreenOptions (CompPlugin *plugin,
 		       CompScreen *screen,
 		       int	  *count)
 {
@@ -509,7 +509,7 @@ getSlotAreas (CompScreen *s)
     if (!slotAreas)
     {
 	free (size);
-        return NULL;
+	return NULL;
     }
 
     left = ss->nWindows;
@@ -1200,6 +1200,7 @@ scaleInitiateCommon (CompScreen      *s,
 
 	sd->lastActiveWindow = s->display->activeWindow;
 	sd->selectedWindow   = s->display->activeWindow;
+	sd->hoveredWindow    = None;
 
 	ss->state = SCALE_STATE_OUT;
 
@@ -1915,12 +1916,18 @@ static const CompMetadataOptionInfo scaleDisplayOptionInfo[] = {
     { "abi", "int", 0, 0, 0 },
     { "index", "int", 0, 0, 0 },
     { "initiate_edge", "edge", 0, scaleInitiate, scaleTerminate },
+    { "initiate_button", "button", 0, scaleInitiate, scaleTerminate },
     { "initiate_key", "key", 0, scaleInitiate, scaleTerminate },
     { "initiate_all_edge", "edge", 0, scaleInitiateAll, scaleTerminate },
+    { "initiate_all_button", "button", 0, scaleInitiateAll, scaleTerminate },
     { "initiate_all_key", "key", 0, scaleInitiateAll, scaleTerminate },
     { "initiate_group_edge", "edge", 0, scaleInitiateGroup, scaleTerminate },
+    { "initiate_group_button", "button", 0,
+      scaleInitiateGroup, scaleTerminate },
     { "initiate_group_key", "key", 0, scaleInitiateGroup, scaleTerminate },
     { "initiate_output_edge", "edge", 0, scaleInitiateOutput, scaleTerminate },
+    { "initiate_output_button", "button", 0,
+      scaleInitiateOutput, scaleTerminate },
     { "initiate_output_key", "key", 0, scaleInitiateOutput, scaleTerminate },
     { "show_desktop", "bool", 0, 0, 0 },
     { "relayout_slots", "action", 0, scaleRelayoutSlots, 0 }
@@ -1932,6 +1939,9 @@ scaleInitDisplay (CompPlugin  *p,
 {
     ScaleDisplay *sd;
 
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
+	return FALSE;
+
     sd = malloc (sizeof (ScaleDisplay));
     if (!sd)
 	return FALSE;
@@ -1968,7 +1978,7 @@ scaleInitDisplay (CompPlugin  *p,
 
     WRAP (sd, d, handleEvent, scaleHandleEvent);
 
-    d->privates[scaleDisplayPrivateIndex].ptr = sd;
+    d->base.privates[scaleDisplayPrivateIndex].ptr = sd;
 
     return TRUE;
 }
@@ -2062,7 +2072,7 @@ scaleInitScreen (CompPlugin *p,
 
     ss->cursor = XCreateFontCursor (s->display->display, XC_left_ptr);
 
-    s->privates[sd->screenPrivateIndex].ptr = ss;
+    s->base.privates[sd->screenPrivateIndex].ptr = ss;
 
     return TRUE;
 }
@@ -2090,6 +2100,8 @@ scaleFiniScreen (CompPlugin *p,
     if (ss->windowsSize)
 	free (ss->windows);
 
+    freeWindowPrivateIndex (s, ss->windowPrivateIndex);
+
     compFiniScreenOptions (s, ss->opt, SCALE_SCREEN_OPTION_NUM);
 
     free (ss);
@@ -2116,7 +2128,7 @@ scaleInitWindow (CompPlugin *p,
     sw->delta = 1.0f;
     sw->lastThumbOpacity = 0.0f;
 
-    w->privates[ss->windowPrivateIndex].ptr = sw;
+    w->base.privates[ss->windowPrivateIndex].ptr = sw;
 
     return TRUE;
 }
@@ -2130,6 +2142,65 @@ scaleFiniWindow (CompPlugin *p,
     free (sw);
 }
 
+static CompBool
+scaleInitObject (CompPlugin *p,
+		 CompObject *o)
+{
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) 0, /* InitCore */
+	(InitPluginObjectProc) scaleInitDisplay,
+	(InitPluginObjectProc) scaleInitScreen,
+	(InitPluginObjectProc) scaleInitWindow
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+scaleFiniObject (CompPlugin *p,
+		 CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) 0, /* FiniCore */
+	(FiniPluginObjectProc) scaleFiniDisplay,
+	(FiniPluginObjectProc) scaleFiniScreen,
+	(FiniPluginObjectProc) scaleFiniWindow
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
+}
+
+static CompOption *
+scaleGetObjectOptions (CompPlugin *plugin,
+		       CompObject *object,
+		       int	  *count)
+{
+    static GetPluginObjectOptionsProc dispTab[] = {
+	(GetPluginObjectOptionsProc) 0, /* GetCoreOptions */
+	(GetPluginObjectOptionsProc) scaleGetDisplayOptions,
+	(GetPluginObjectOptionsProc) scaleGetScreenOptions
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
+		     (void *) (*count = 0), (plugin, object, count));
+}
+
+static CompBool
+scaleSetObjectOption (CompPlugin      *plugin,
+		      CompObject      *object,
+		      const char      *name,
+		      CompOptionValue *value)
+{
+    static SetPluginObjectOptionProc dispTab[] = {
+	(SetPluginObjectOptionProc) 0, /* SetCoreOption */
+	(SetPluginObjectOptionProc) scaleSetDisplayOption,
+	(SetPluginObjectOptionProc) scaleSetScreenOption
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab), FALSE,
+		     (plugin, object, name, value));
+}
+
 static Bool
 scaleInit (CompPlugin *p)
 {
@@ -2160,13 +2231,6 @@ scaleFini (CompPlugin *p)
     compFiniMetadata (&scaleMetadata);
 }
 
-static int
-scaleGetVersion (CompPlugin *plugin,
-		 int	    version)
-{
-    return ABIVERSION;
-}
-
 static CompMetadata *
 scaleGetMetadata (CompPlugin *plugin)
 {
@@ -2175,24 +2239,17 @@ scaleGetMetadata (CompPlugin *plugin)
 
 CompPluginVTable scaleVTable = {
     "scale",
-    scaleGetVersion,
     scaleGetMetadata,
     scaleInit,
     scaleFini,
-    scaleInitDisplay,
-    scaleFiniDisplay,
-    scaleInitScreen,
-    scaleFiniScreen,
-    scaleInitWindow,
-    scaleFiniWindow,
-    scaleGetDisplayOptions,
-    scaleSetDisplayOption,
-    scaleGetScreenOptions,
-    scaleSetScreenOption
+    scaleInitObject,
+    scaleFiniObject,
+    scaleGetObjectOptions,
+    scaleSetObjectOption
 };
 
 CompPluginVTable *
-getCompPluginInfo (void)
+getCompPluginInfo20070830 (void)
 {
     return &scaleVTable;
 }
