@@ -33,7 +33,7 @@
 #include <X11/Xatom.h>
 #include <X11/extensions/shape.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 #include <decoration.h>
 
 static CompMetadata svgMetadata;
@@ -94,20 +94,20 @@ typedef struct _SvgWindow {
     SvgContext *context;
 } SvgWindow;
 
-#define GET_SVG_DISPLAY(d)				    \
-    ((SvgDisplay *) (d)->privates[displayPrivateIndex].ptr)
+#define GET_SVG_DISPLAY(d)					 \
+    ((SvgDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 
 #define SVG_DISPLAY(d)			 \
     SvgDisplay *sd = GET_SVG_DISPLAY (d)
 
-#define GET_SVG_SCREEN(s, sd)					\
-    ((SvgScreen *) (s)->privates[(sd)->screenPrivateIndex].ptr)
+#define GET_SVG_SCREEN(s, sd)					     \
+    ((SvgScreen *) (s)->base.privates[(sd)->screenPrivateIndex].ptr)
 
 #define SVG_SCREEN(s)						     \
     SvgScreen *ss = GET_SVG_SCREEN (s, GET_SVG_DISPLAY (s->display))
 
-#define GET_SVG_WINDOW(w, ss)					\
-    ((SvgWindow *) (w)->privates[(ss)->windowPrivateIndex].ptr)
+#define GET_SVG_WINDOW(w, ss)					     \
+    ((SvgWindow *) (w)->base.privates[(ss)->windowPrivateIndex].ptr)
 
 #define SVG_WINDOW(w)					   \
     SvgWindow *sw = GET_SVG_WINDOW  (w,			   \
@@ -797,6 +797,9 @@ svgInitDisplay (CompPlugin  *p,
     SvgDisplay *sd;
     CompScreen *s;
 
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
+	return FALSE;
+
     sd = malloc (sizeof (SvgDisplay));
     if (!sd)
 	return FALSE;
@@ -822,7 +825,7 @@ svgInitDisplay (CompPlugin  *p,
     WRAP (sd, d, handleCompizEvent, svgHandleCompizEvent);
     WRAP (sd, d, fileToImage, svgFileToImage);
 
-    d->privates[displayPrivateIndex].ptr = sd;
+    d->base.privates[displayPrivateIndex].ptr = sd;
 
     for (s = d->screens; s; s = s->next)
 	updateDefaultIcon (s);
@@ -876,7 +879,7 @@ svgInitScreen (CompPlugin *p,
     WRAP (ss, s, windowMoveNotify, svgWindowMoveNotify);
     WRAP (ss, s, windowResizeNotify, svgWindowResizeNotify);
 
-    s->privates[sd->screenPrivateIndex].ptr = ss;
+    s->base.privates[sd->screenPrivateIndex].ptr = ss;
 
     return TRUE;
 }
@@ -911,7 +914,7 @@ svgInitWindow (CompPlugin *p,
     sw->source  = NULL;
     sw->context = NULL;
 
-    w->privates[ss->windowPrivateIndex].ptr = sw;
+    w->base.privates[ss->windowPrivateIndex].ptr = sw;
 
     return TRUE;
 }
@@ -937,6 +940,63 @@ svgFiniWindow (CompPlugin *p,
     free (sw);
 }
 
+static CompBool
+svgInitObject (CompPlugin *p,
+	       CompObject *o)
+{
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) 0, /* InitCore */
+	(InitPluginObjectProc) svgInitDisplay,
+	(InitPluginObjectProc) svgInitScreen,
+	(InitPluginObjectProc) svgInitWindow
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+svgFiniObject (CompPlugin *p,
+	       CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) 0, /* FiniCore */
+	(FiniPluginObjectProc) svgFiniDisplay,
+	(FiniPluginObjectProc) svgFiniScreen,
+	(FiniPluginObjectProc) svgFiniWindow
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
+}
+
+static CompOption *
+svgGetObjectOptions (CompPlugin *plugin,
+		     CompObject *object,
+		     int	*count)
+{
+    static GetPluginObjectOptionsProc dispTab[] = {
+	(GetPluginObjectOptionsProc) 0, /* GetCoreOptions */
+	(GetPluginObjectOptionsProc) svgGetDisplayOptions
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab),
+		     (void *) (*count = 0), (plugin, object, count));
+}
+
+static CompBool
+svgSetObjectOption (CompPlugin      *plugin,
+		    CompObject      *object,
+		    const char      *name,
+		    CompOptionValue *value)
+{
+    static SetPluginObjectOptionProc dispTab[] = {
+	(SetPluginObjectOptionProc) 0, /* SetCoreOption */
+	(SetPluginObjectOptionProc) svgSetDisplayOption
+    };
+
+    RETURN_DISPATCH (object, dispTab, ARRAY_SIZE (dispTab), FALSE,
+		     (plugin, object, name, value));
+}
+
 static Bool
 svgInit (CompPlugin *p)
 {
@@ -954,6 +1014,8 @@ svgInit (CompPlugin *p)
 	return FALSE;
     }
 
+    rsvg_init ();
+
     compAddMetadataFromFile (&svgMetadata, p->vTable->name);
 
     return TRUE;
@@ -964,13 +1026,8 @@ svgFini (CompPlugin *p)
 {
     freeDisplayPrivateIndex (displayPrivateIndex);
     compFiniMetadata (&svgMetadata);
-}
 
-static int
-svgGetVersion (CompPlugin *plugin,
-	       int	  version)
-{
-    return ABIVERSION;
+    rsvg_term ();
 }
 
 static CompMetadata *
@@ -981,24 +1038,17 @@ svgGetMetadata (CompPlugin *plugin)
 
 CompPluginVTable svgVTable = {
     "svg",
-    svgGetVersion,
     svgGetMetadata,
     svgInit,
     svgFini,
-    svgInitDisplay,
-    svgFiniDisplay,
-    svgInitScreen,
-    svgFiniScreen,
-    svgInitWindow,
-    svgFiniWindow,
-    svgGetDisplayOptions,
-    svgSetDisplayOption,
-    0, /* GetScreenOptions */
-    0  /* SetScreenOption */
+    svgInitObject,
+    svgFiniObject,
+    svgGetObjectOptions,
+    svgSetObjectOption
 };
 
 CompPluginVTable *
-getCompPluginInfo (void)
+getCompPluginInfo20070830 (void)
 {
     return &svgVTable;
 }
