@@ -32,7 +32,7 @@
 #include <X11/extensions/Xrandr.h>
 #include <X11/extensions/Xfixes.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 
 static Window xdndWindow = None;
 static Window edgeWindow = None;
@@ -139,7 +139,7 @@ moveInputFocusToOtherWindow (CompWindow *w)
 		moveInputFocusToWindow (ancestor);
 	    }
 	    else
-		focusDefaultWindow (display);
+		focusDefaultWindow (w->screen);
 	}
 	else if (w->type & (CompWindowTypeDialogMask |
 			    CompWindowTypeModalDialogMask))
@@ -174,10 +174,10 @@ moveInputFocusToOtherWindow (CompWindow *w)
 		moveInputFocusToWindow (focus);
 	    }
 	    else
-		focusDefaultWindow (display);
+		focusDefaultWindow (w->screen);
 	}
 	else
-	    focusDefaultWindow (display);
+	    focusDefaultWindow (w->screen);
     }
 }
 
@@ -185,13 +185,11 @@ static Bool
 autoRaiseTimeout (void *closure)
 {
     CompDisplay *display = closure;
-    CompWindow *w = findWindowAtDisplay (display, display->activeWindow);
+    CompWindow  *w = findWindowAtDisplay (display, display->activeWindow);
 
     if (display->autoRaiseWindow == display->activeWindow ||
-	display->autoRaiseWindow == w->transientFor)
+	(w && (display->autoRaiseWindow == w->transientFor)))
     {
-	CompWindow *w;
-
 	w = findWindowAtDisplay (display, display->autoRaiseWindow);
 	if (w)
 	    updateWindowAttributes (w, CompStackingUpdateModeNormal);
@@ -208,7 +206,7 @@ isCallBackBinding (CompOption	   *option,
 		   CompBindingType type,
 		   CompActionState state)
 {
-    if (option->type != CompOptionTypeAction)
+    if (!isActionOption (option))
 	return FALSE;
 
     if (!(option->value.action.type & type))
@@ -314,9 +312,14 @@ triggerButtonPressBindings (CompDisplay *d,
 	    if (isInitiateBinding (option, CompBindingTypeEdgeButton,
 				   CompActionStateInitEdge, &action))
 	    {
-		if (action->edgeMask & edge)
+		if ((action->button.button == event->xbutton.button) &&
+		    (action->edgeMask & edge))
 		{
-		    if (action->edgeButton == event->xbutton.button)
+		    bindMods = virtualToRealModMask (d,
+						     action->button.modifiers);
+
+		    if ((bindMods & modMask) ==
+			(event->xbutton.state & modMask))
 			if ((*action->initiate) (d, action,
 						 CompActionStateInitEdge,
 						 argument, nArgument))
@@ -385,7 +388,7 @@ triggerKeyPressBindings (CompDisplay *d,
 
 	while (n--)
 	{
-	    if (o->type == CompOptionTypeAction)
+	    if (isActionOption (o))
 	    {
 		if (o->value.action.terminate)
 		    (*o->value.action.terminate) (d, &o->value.action,
@@ -536,7 +539,8 @@ isBellAction (CompOption      *option,
 	      CompActionState state,
 	      CompAction      **action)
 {
-    if (option->type != CompOptionTypeAction)
+    if (option->type != CompOptionTypeAction &&
+	option->type != CompOptionTypeBell)
 	return FALSE;
 
     if (!option->value.action.bell)
@@ -582,7 +586,9 @@ isEdgeAction (CompOption      *option,
 	      CompActionState state,
 	      unsigned int    edge)
 {
-    if (option->type != CompOptionTypeAction)
+    if (option->type != CompOptionTypeAction &&
+	option->type != CompOptionTypeButton &&
+	option->type != CompOptionTypeEdge)
 	return FALSE;
 
     if (!(option->value.action.edgeMask & edge))
@@ -685,6 +691,7 @@ static Bool
 handleActionEvent (CompDisplay *d,
 		   XEvent      *event)
 {
+    CompObject *obj = &d->base;
     CompOption *option;
     int	       nOption;
     CompPlugin *p;
@@ -727,19 +734,13 @@ handleActionEvent (CompDisplay *d,
 
 	for (p = getPlugins (); p; p = p->next)
 	{
-	    if (p->vTable->getDisplayOptions)
-	    {
-		option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-		if (triggerButtonPressBindings (d, option, nOption, event,
-						o, 8))
-		    return TRUE;
-	    }
+	    if (!p->vTable->getObjectOptions)
+		continue;
+
+	    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+	    if (triggerButtonPressBindings (d, option, nOption, event, o, 8))
+		return TRUE;
 	}
-
-	option = compGetDisplayOptions (d, &nOption);
-	if (triggerButtonPressBindings (d, option, nOption, event, o, 8))
-	    return TRUE;
-
 	break;
     case ButtonRelease:
 	o[0].value.i = event->xbutton.window;
@@ -759,19 +760,13 @@ handleActionEvent (CompDisplay *d,
 
 	for (p = getPlugins (); p; p = p->next)
 	{
-	    if (p->vTable->getDisplayOptions)
-	    {
-		option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-		if (triggerButtonReleaseBindings (d, option, nOption, event,
-						  o, 8))
-		    return TRUE;
-	    }
+	    if (!p->vTable->getObjectOptions)
+		continue;
+
+	    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+	    if (triggerButtonReleaseBindings (d, option, nOption, event, o, 8))
+		return TRUE;
 	}
-
-	option = compGetDisplayOptions (d, &nOption);
-	if (triggerButtonReleaseBindings (d, option, nOption, event, o, 8))
-	    return TRUE;
-
 	break;
     case KeyPress:
 	o[0].value.i = event->xkey.window;
@@ -791,18 +786,13 @@ handleActionEvent (CompDisplay *d,
 
 	for (p = getPlugins (); p; p = p->next)
 	{
-	    if (p->vTable->getDisplayOptions)
-	    {
-		option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-		if (triggerKeyPressBindings (d, option, nOption, event, o, 8))
-		    return TRUE;
-	    }
+	    if (!p->vTable->getObjectOptions)
+		continue;
+
+	    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+	    if (triggerKeyPressBindings (d, option, nOption, event, o, 8))
+		return TRUE;
 	}
-
-	option = compGetDisplayOptions (d, &nOption);
-	if (triggerKeyPressBindings (d, option, nOption, event, o, 8))
-	    return TRUE;
-
 	break;
     case KeyRelease:
 	o[0].value.i = event->xkey.window;
@@ -822,18 +812,12 @@ handleActionEvent (CompDisplay *d,
 
 	for (p = getPlugins (); p; p = p->next)
 	{
-	    if (p->vTable->getDisplayOptions)
-	    {
-		option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-		if (triggerKeyReleaseBindings (d, option, nOption, event, o, 8))
-		    return TRUE;
-	    }
+	    if (!p->vTable->getObjectOptions)
+		continue;
+	    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+	    if (triggerKeyReleaseBindings (d, option, nOption, event, o, 8))
+		return TRUE;
 	}
-
-	option = compGetDisplayOptions (d, &nOption);
-	if (triggerKeyReleaseBindings (d, option, nOption, event, o, 8))
-	    return TRUE;
-
 	break;
     case EnterNotify:
 	if (event->xcrossing.mode   != NotifyGrab   &&
@@ -877,19 +861,14 @@ handleActionEvent (CompDisplay *d,
 
 		for (p = getPlugins (); p; p = p->next)
 		{
-		    if (p->vTable->getDisplayOptions)
-		    {
-			option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-			if (triggerEdgeLeaveBindings (d, option, nOption, state,
-						      edge, o, 7))
-			    return TRUE;
-		    }
-		}
+		    if (!p->vTable->getObjectOptions)
+			continue;
 
-		option = compGetDisplayOptions (d, &nOption);
-		if (triggerEdgeLeaveBindings (d, option, nOption, state,
-					      edge, o, 7))
-		    return TRUE;
+		    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+		    if (triggerEdgeLeaveBindings (d, option, nOption, state,
+						  edge, o, 7))
+			return TRUE;
+		}
 	    }
 
 	    edge = 0;
@@ -922,19 +901,14 @@ handleActionEvent (CompDisplay *d,
 
 		for (p = getPlugins (); p; p = p->next)
 		{
-		    if (p->vTable->getDisplayOptions)
-		    {
-			option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-			if (triggerEdgeEnterBindings (d, option, nOption, state,
-						      edge, o, 7))
-			    return TRUE;
-		    }
-		}
+		    if (!p->vTable->getObjectOptions)
+			continue;
 
-		option = compGetDisplayOptions (d, &nOption);
-		if (triggerEdgeEnterBindings (d, option, nOption, state,
-					      edge, o, 7))
-		    return TRUE;
+		    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+		    if (triggerEdgeEnterBindings (d, option, nOption, state,
+						  edge, o, 7))
+			return TRUE;
+		}
 	    }
 	} break;
     case ClientMessage:
@@ -983,19 +957,14 @@ handleActionEvent (CompDisplay *d,
 
 		for (p = getPlugins (); p; p = p->next)
 		{
-		    if (p->vTable->getDisplayOptions)
-		    {
-			option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-			if (triggerEdgeLeaveBindings (d, option, nOption, state,
-						      edge, o, 6))
-			    return TRUE;
-		    }
-		}
+		    if (!p->vTable->getObjectOptions)
+			continue;
 
-		option = compGetDisplayOptions (d, &nOption);
-		if (triggerEdgeLeaveBindings (d, option, nOption, state,
-					      edge, o, 6))
-		    return TRUE;
+		    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+		    if (triggerEdgeLeaveBindings (d, option, nOption, state,
+						  edge, o, 6))
+			return TRUE;
+		}
 	    }
 	}
 	else if (event->xclient.message_type == d->xdndPositionAtom)
@@ -1039,19 +1008,14 @@ handleActionEvent (CompDisplay *d,
 
 		for (p = getPlugins (); p; p = p->next)
 		{
-		    if (p->vTable->getDisplayOptions)
-		    {
-			option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-			if (triggerEdgeEnterBindings (d, option, nOption, state,
-						      edge, o, 6))
-			    return TRUE;
-		    }
-		}
+		    if (!p->vTable->getObjectOptions)
+			continue;
 
-		option = compGetDisplayOptions (d, &nOption);
-		if (triggerEdgeEnterBindings (d, option, nOption, state,
-					      edge, o, 6))
-		    return TRUE;
+		    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+		    if (triggerEdgeEnterBindings (d, option, nOption, state,
+						  edge, o, 6))
+			return TRUE;
+		}
 	    }
 
 	    xdndWindow = None;
@@ -1077,8 +1041,6 @@ handleActionEvent (CompDisplay *d,
 	    {
 		XkbStateNotifyEvent *stateEvent = (XkbStateNotifyEvent *) event;
 
-		option = compGetDisplayOptions (d, &nOption);
-
 		o[0].value.i = d->activeWindow;
 		o[1].value.i = d->activeWindow;
 		o[2].value.i = stateEvent->mods;
@@ -1089,24 +1051,17 @@ handleActionEvent (CompDisplay *d,
 
 		for (p = getPlugins (); p; p = p->next)
 		{
-		    if (p->vTable->getDisplayOptions)
-		    {
-			option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-			if (triggerStateNotifyBindings (d, option, nOption,
-							stateEvent, o, 4))
-			    return TRUE;
-		    }
-		}
+		    if (!p->vTable->getObjectOptions)
+			continue;
 
-		option = compGetDisplayOptions (d, &nOption);
-		if (triggerStateNotifyBindings (d, option, nOption, stateEvent,
-						o, 4))
-		    return TRUE;
+		    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+		    if (triggerStateNotifyBindings (d, option, nOption,
+						    stateEvent, o, 4))
+			return TRUE;
+		}
 	    }
 	    else if (xkbEvent->xkb_type == XkbBellNotify)
 	    {
-		option = compGetDisplayOptions (d, &nOption);
-
 		o[0].value.i = d->activeWindow;
 		o[1].value.i = d->activeWindow;
 
@@ -1116,18 +1071,13 @@ handleActionEvent (CompDisplay *d,
 
 		for (p = getPlugins (); p; p = p->next)
 		{
-		    if (p->vTable->getDisplayOptions)
-		    {
-			option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-			if (triggerBellNotifyBindings (d, option, nOption,
-						       o, 3))
-			    return TRUE;
-		    }
-		}
+		    if (!p->vTable->getObjectOptions)
+			continue;
 
-		option = compGetDisplayOptions (d, &nOption);
-		if (triggerBellNotifyBindings (d, option, nOption, o, 3))
-		    return TRUE;
+		    option = (*p->vTable->getObjectOptions) (p, obj, &nOption);
+		    if (triggerBellNotifyBindings (d, option, nOption, o, 3))
+			return TRUE;
+		}
 	    }
 	}
 	break;
@@ -1138,8 +1088,8 @@ handleActionEvent (CompDisplay *d,
 
 void
 handleCompizEvent (CompDisplay *d,
-		   char        *pluginName,
-		   char        *eventName,
+		   const char  *pluginName,
+		   const char  *eventName,
 		   CompOption  *option,
 		   int         nOption)
 {
@@ -1364,7 +1314,7 @@ handleEvent (CompDisplay *d,
 		if (w)
 		{
 		    if (d->opt[COMP_DISPLAY_OPTION_RAISE_ON_CLICK].value.b)
-			updateWindowAttributes (w, 
+			updateWindowAttributes (w,
 					CompStackingUpdateModeAboveFullscreen);
 
 		    if (!(w->type & CompWindowTypeDockMask))
@@ -1740,11 +1690,11 @@ handleEvent (CompDisplay *d,
 
 		value.i = event->xclient.data.l[0] / s->width;
 
-		(*s->setScreenOption) (s, "hsize", &value);
+		(*core.setOptionForPlugin) (&s->base, "core", "hsize", &value);
 
 		value.i = event->xclient.data.l[1] / s->height;
 
-		(*s->setScreenOption) (s, "vsize", &value);
+		(*core.setOptionForPlugin) (&s->base, "core", "vsize", &value);
 	    }
 	}
 	else if (event->xclient.message_type == d->moveResizeWindowAtom)
@@ -1818,7 +1768,7 @@ handleEvent (CompDisplay *d,
 	else if (event->xclient.message_type == d->wmChangeStateAtom)
 	{
 	    w = findWindowAtDisplay (d, event->xclient.window);
-	    if (w && w->type & CompWindowTypeNormalMask)
+	    if (w)
 	    {
 		if (event->xclient.data.l[0] == IconicState)
 		    minimizeWindow (w);
@@ -1849,7 +1799,9 @@ handleEvent (CompDisplay *d,
 
 		value.i = event->xclient.data.l[0];
 
-		(*s->setScreenOption) (s, "number_of_desktops", &value);
+		(*core.setOptionForPlugin) (&s->base,
+					    "core", "number_of_desktops",
+					    &value);
 	    }
 	}
 	else if (event->xclient.message_type == d->currentDesktopAtom)
@@ -1897,7 +1849,8 @@ handleEvent (CompDisplay *d,
 
 	    if (doMapProcessing)
 	    {
-		Bool allowFocus;
+		Bool                   allowFocus;
+		CompStackingUpdateMode stackingMode;
 
 		w->initialViewportX = w->screen->x;
 		w->initialViewportY = w->screen->y;
@@ -1925,23 +1878,12 @@ handleEvent (CompDisplay *d,
 
 		allowFocus = allowWindowFocus (w, NO_FOCUS_MASK, 0);
 
-		updateWindowAttributes (w, CompStackingUpdateModeInitialMap);
+		if (!allowFocus && (w->type & ~NO_FOCUS_MASK))
+		    stackingMode = CompStackingUpdateModeInitialMapDeniedFocus;
+		else
+		    stackingMode = CompStackingUpdateModeInitialMap;
 
-		if (!allowFocus &&
-		    (w->type & ~(CompWindowTypeSplashMask |
-				 CompWindowTypeDockMask   |
-				 CompWindowTypeDesktopMask)))
-		{
-		    CompWindow *p;
-
-		    for (p = w->prev; p; p = p->prev)
-			if (p->id == d->activeWindow)
-			    break;
-
-		    /* window is above active window so we should lower it */
-		    if (p)
-			restackWindowBelow (w, p);
-		}
+		updateWindowAttributes (w, stackingMode);
 
 		if (w->minimized)
 		    unminimizeWindow (w);
@@ -2066,9 +2008,7 @@ handleEvent (CompDisplay *d,
 		}
 
 		state &= ~CompWindowStateDemandsAttentionMask;
-
-		if (w->state != state)
-		    changeWindowState (w, state);
+		changeWindowState (w, state);
 	    }
 	}
 	break;
