@@ -34,7 +34,7 @@
 #include <string.h>
 #include <sys/wait.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 
 char *programName;
 char **programArgv;
@@ -100,9 +100,9 @@ usage (void)
 
 void
 compLogMessage (CompDisplay *d,
-		char	     *componentName,
+		const char   *componentName,
 		CompLogLevel level,
-		char	     *format,
+		const char   *format,
 		...)
 {
     va_list args;
@@ -122,9 +122,9 @@ compLogMessage (CompDisplay *d,
 
 void
 logMessage (CompDisplay	 *d,
-	    char	 *componentName,
+	    const char	 *componentName,
 	    CompLogLevel level,
-	    char	 *message)
+	    const char	 *message)
 {
     char defaultMessage[2048];
 
@@ -264,8 +264,6 @@ main (int argc, char **argv)
     programArgc = argc;
     programArgv = argv;
 
-    compDisplays = NULL;
-
     signal (SIGHUP, signalHandler);
     signal (SIGCHLD, signalHandler);
     signal (SIGINT, signalHandler);
@@ -432,6 +430,9 @@ main (int argc, char **argv)
 
     compAddMetadataFromFile (&coreMetadata, "core");
 
+    if (!initCore ())
+	return 1;
+
     if (!disableSm)
 	initSession (clientId);
 
@@ -443,6 +444,8 @@ main (int argc, char **argv)
     if (!disableSm)
 	closeSession ();
 
+    finiCore ();
+
     xmlCleanupParser ();
 
     if (restartSignal)
