@@ -30,7 +30,7 @@
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 
 #define HOME_METADATADIR ".compiz/metadata"
 #define EXTENSION ".xml"
@@ -1119,6 +1119,7 @@ compFiniScreenOption (CompScreen *s,
 {
     finiScreenOptionValue (s, &o->value, o->type);
     compFiniOption (o);
+    free (o->name);
 }
 
 Bool
@@ -1215,6 +1216,7 @@ compFiniDisplayOption (CompDisplay *d,
 {
     finiDisplayOptionValue (d, &o->value, o->type);
     compFiniOption (o);
+    free (o->name);
 }
 
 Bool
