@@ -27,7 +27,7 @@
 #include <string.h>
 #include <math.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 
 ScreenPaintAttrib defaultScreenPaintAttrib = {
     0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, -DEFAULT_Z_CAMERA
@@ -39,15 +39,19 @@ WindowPaintAttrib defaultWindowPaintAttrib = {
 
 void
 preparePaintScreen (CompScreen *screen,
-		    int	       msSinceLastPaint) {}
+		    int	       msSinceLastPaint)
+{
+}
 
 void
-donePaintScreen (CompScreen *screen) {}
+donePaintScreen (CompScreen *screen)
+{
+}
 
 void
 applyScreenTransform (CompScreen	      *screen,
 		      const ScreenPaintAttrib *sAttrib,
-		      int		      output,
+		      CompOutput	      *output,
 		      CompTransform	      *transform)
 {
     matrixTranslate (transform,
@@ -58,144 +62,139 @@ applyScreenTransform (CompScreen	      *screen,
 		  sAttrib->xRotate, 0.0f, 1.0f, 0.0f);
     matrixRotate (transform,
 		  sAttrib->vRotate,
-		  1.0f - sAttrib->xRotate / 90.0f,
+		  cosf (sAttrib->xRotate * DEG2RAD),
 		  0.0f,
-		  sAttrib->xRotate / 90.0f);
+		  sinf (sAttrib->xRotate * DEG2RAD));
     matrixRotate (transform,
 		  sAttrib->yRotate, 0.0f, 1.0f, 0.0f);
 }
 
 void
 transformToScreenSpace (CompScreen    *screen,
-			int	      output,
+			CompOutput    *output,
 			float         z,
 			CompTransform *transform)
 {
     matrixTranslate (transform, -0.5f, -0.5f, z);
     matrixScale (transform,
-		 1.0f  / screen->outputDev[output].width,
-		 -1.0f / screen->outputDev[output].height,
+		 1.0f  / output->width,
+		 -1.0f / output->height,
 		 1.0f);
     matrixTranslate (transform,
-		     -screen->outputDev[output].region.extents.x1,
-		     -screen->outputDev[output].region.extents.y2,
+		     -output->region.extents.x1,
+		     -output->region.extents.y2,
 		     0.0f);
 }
 
 void
 prepareXCoords (CompScreen *screen,
-		int	   output,
+		CompOutput *output,
 		float      z)
 {
     glTranslatef (-0.5f, -0.5f, z);
-    glScalef (1.0f  / screen->outputDev[output].width,
-	      -1.0f / screen->outputDev[output].height,
+    glScalef (1.0f  / output->width,
+	      -1.0f / output->height,
 	      1.0f);
-    glTranslatef (-screen->outputDev[output].region.extents.x1,
-		  -screen->outputDev[output].region.extents.y2,
+    glTranslatef (-output->region.extents.x1,
+		  -output->region.extents.y2,
 		  0.0f);
 }
 
 void
-paintTransformedScreen (CompScreen		*screen,
-			const ScreenPaintAttrib *sAttrib,
-			const CompTransform	*transform,
-			Region			region,
-			int			output,
-			unsigned int		mask)
+paintCursor (CompCursor		 *c,
+	     const CompTransform *transform,
+	     Region		 region,
+	     unsigned int	 mask)
 {
-    CompTransform sTransform = *transform;
-    CompWindow	  *w;
-    int	          windowMask;
-    int	          backgroundMask;
+    int x1, y1, x2, y2;
 
-    if (mask & PAINT_SCREEN_CLEAR_MASK)
-	clearTargetOutput (screen->display, GL_COLOR_BUFFER_BIT);
+    if (!c->image)
+	return;
 
-    screenLighting (screen, TRUE);
+    x1 = c->x;
+    y1 = c->y;
+    x2 = c->x + c->image->width;
+    y2 = c->y + c->image->height;
 
-    (*screen->applyScreenTransform) (screen, sAttrib, output, &sTransform);
+    glDisableClientState (GL_TEXTURE_COORD_ARRAY);
+    glEnable (GL_BLEND);
+
+    enableTexture (c->screen, &c->image->texture, COMP_TEXTURE_FILTER_FAST);
+
+    glBegin (GL_QUADS);
+
+    glTexCoord2f (COMP_TEX_COORD_X (&c->matrix, x1),
+		  COMP_TEX_COORD_Y (&c->matrix, y2));
+    glVertex2i (x1, y2);
+    glTexCoord2f (COMP_TEX_COORD_X (&c->matrix, x2),
+		  COMP_TEX_COORD_Y (&c->matrix, y2));
+    glVertex2i (x2, y2);
+    glTexCoord2f (COMP_TEX_COORD_X (&c->matrix, x2),
+		  COMP_TEX_COORD_Y (&c->matrix, y1));
+    glVertex2i (x2, y1);
+    glTexCoord2f (COMP_TEX_COORD_X (&c->matrix, x1),
+		  COMP_TEX_COORD_Y (&c->matrix, y1));
+    glVertex2i (x1, y1);
+
+    glEnd ();
+
+    disableTexture (c->screen, &c->image->texture);
+
+    glDisable (GL_BLEND);
+    glEnableClientState (GL_TEXTURE_COORD_ARRAY);
+}
+
+/* This function currently always performs occlusion detection to
+   minimize paint regions. OpenGL precision requirements are no good
+   enough to guarantee that the results from using occlusion detection
+   is the same as without. It's likely not possible to see any
+   difference with most hardware but occlusion detection in the
+   transformed screen case should be made optional for those who do
+   see a difference. */
+static void
+paintOutputRegion (CompScreen	       *screen,
+		   const CompTransform *transform,
+		   Region	       region,
+		   CompOutput	       *output,
+		   unsigned int	       mask)
+{
+    static Region tmpRegion = NULL;
+    CompWindow    *w;
+    CompCursor	  *c;
+    int		  count, windowMask, backgroundMask, odMask;
+    CompWindow	  *fullscreenWindow = NULL;
+    CompWalker    walk;
+    Bool          status;
+    Bool          withOffset = FALSE;
+    CompTransform vTransform;
+    int           offX, offY;
+
+    if (!tmpRegion)
+    {
+	tmpRegion = XCreateRegion ();
+	if (!tmpRegion)
+	    return;
+    }
 
     if (mask & PAINT_SCREEN_TRANSFORMED_MASK)
     {
-	windowMask = PAINT_WINDOW_ON_TRANSFORMED_SCREEN_MASK;
 	backgroundMask = PAINT_BACKGROUND_ON_TRANSFORMED_SCREEN_MASK;
-
-	if (mask & PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_MASK)
-	{
-	    GLdouble h = screen->height;
-
-	    GLdouble p1[2] = { region->extents.x1, h - region->extents.y2 };
-	    GLdouble p2[2] = { region->extents.x2, h - region->extents.y1 };
-
-	    GLdouble halfW = screen->outputDev[output].width / 2.0;
-	    GLdouble halfH = screen->outputDev[output].height / 2.0;
-
-	    GLdouble cx = screen->outputDev[output].region.extents.x1 + halfW;
-	    GLdouble cy = (h - screen->outputDev[output].region.extents.y2) +
-		halfH;
-
-	    GLdouble top[4]    = { 0.0, halfH / (cy - p1[1]), 0.0, 0.5 };
-	    GLdouble bottom[4] = { 0.0, halfH / (cy - p2[1]), 0.0, 0.5 };
-	    GLdouble left[4]   = { halfW / (cx - p1[0]), 0.0, 0.0, 0.5 };
-	    GLdouble right[4]  = { halfW / (cx - p2[0]), 0.0, 0.0, 0.5 };
-
-	    glPushMatrix ();
-	    glLoadMatrixf (sTransform.m);
-
-	    glClipPlane (GL_CLIP_PLANE0, top);
-	    glClipPlane (GL_CLIP_PLANE1, bottom);
-	    glClipPlane (GL_CLIP_PLANE2, left);
-	    glClipPlane (GL_CLIP_PLANE3, right);
-
-	    glEnable (GL_CLIP_PLANE0);
-	    glEnable (GL_CLIP_PLANE1);
-	    glEnable (GL_CLIP_PLANE2);
-	    glEnable (GL_CLIP_PLANE3);
-
-	    transformToScreenSpace (screen, output, -sAttrib->zTranslate,
-				    &sTransform);
-
-	    glLoadMatrixf (sTransform.m);
-
-	    (*screen->paintBackground) (screen, region, backgroundMask);
-
-	    for (w = screen->windows; w; w = w->next)
-	    {
-		if (w->destroyed)
-		    continue;
-
-		if (!w->shaded)
-		{
-		    if (w->attrib.map_state != IsViewable || !w->damaged)
-			continue;
-		}
-
-		(*screen->paintWindow) (w, &w->paint, &sTransform, region,
-					windowMask);
-	    }
-
-	    glDisable (GL_CLIP_PLANE0);
-	    glDisable (GL_CLIP_PLANE1);
-	    glDisable (GL_CLIP_PLANE2);
-	    glDisable (GL_CLIP_PLANE3);
-
-	    glPopMatrix ();
-
-	    return;
-	}
+	windowMask     = PAINT_WINDOW_ON_TRANSFORMED_SCREEN_MASK;
+	count	       = 1;
     }
     else
-	windowMask = backgroundMask = 0;
+    {
+	backgroundMask = 0;
+	windowMask     = 0;
+	count	       = 0;
+    }
 
-    transformToScreenSpace (screen, output, -sAttrib->zTranslate, &sTransform);
+    XSubtractRegion (region, &emptyRegion, tmpRegion);
 
-    glPushMatrix ();
-    glLoadMatrixf (sTransform.m);
+    (*screen->initWindowWalker) (screen, &walk);
 
-    (*screen->paintBackground) (screen, region, backgroundMask);
-
-    for (w = screen->windows; w; w = w->next)
+    /* detect occlusions */
+    for (w = (*walk.last) (screen); w; w = (*walk.prev) (w))
     {
 	if (w->destroyed)
 	    continue;
@@ -206,23 +205,209 @@ paintTransformedScreen (CompScreen		*screen,
 		continue;
 	}
 
-	(*screen->paintWindow) (w, &w->paint, &sTransform, region, windowMask);
+	/* copy region */
+	XSubtractRegion (tmpRegion, &emptyRegion, w->clip);
+
+	odMask = PAINT_WINDOW_OCCLUSION_DETECTION_MASK;
+	
+	if ((screen->windowOffsetX != 0 || screen->windowOffsetY != 0) &&
+	    !windowOnAllViewports (w))
+	{
+	    withOffset = TRUE;
+
+	    getWindowMovementForOffset (w, screen->windowOffsetX,
+					screen->windowOffsetY, &offX, &offY);
+
+	    vTransform = *transform;
+	    matrixTranslate (&vTransform, offX, offY, 0);
+ 
+	    XOffsetRegion (w->clip, -offX, -offY);
+
+	    odMask |= PAINT_WINDOW_WITH_OFFSET_MASK;
+	    status = (*screen->paintWindow) (w, &w->paint, &vTransform,
+					     tmpRegion, odMask);
+	}
+	else
+	{
+	    withOffset = FALSE;
+	    status = (*screen->paintWindow) (w, &w->paint, transform, tmpRegion,
+					     odMask);
+	}
+
+	if (status)
+	{
+	    if (withOffset)
+	    {
+		XOffsetRegion (w->region, offX, offY);
+		XSubtractRegion (tmpRegion, w->region, tmpRegion);
+		XOffsetRegion (w->region, -offX, -offY);
+	    }
+	    else
+		XSubtractRegion (tmpRegion, w->region, tmpRegion);
+
+	    /* unredirect top most fullscreen windows. */
+	    if (count == 0					      &&
+		!REGION_NOT_EMPTY (tmpRegion)			      &&
+		screen->opt[COMP_SCREEN_OPTION_UNREDIRECT_FS].value.b &&
+		XEqualRegion (w->region, &screen->region))
+	    {
+		unredirectWindow (w);
+		fullscreenWindow = w;
+	    }
+	}
+
+	count++;
     }
 
+    (*screen->paintBackground) (screen, tmpRegion, backgroundMask);
+
+    /* paint all windows from bottom to top */
+    for (w = (*walk.first) (screen); w; w = (*walk.next) (w))
+    {
+	if (w->destroyed)
+	    continue;
+
+	if (w == fullscreenWindow)
+	    continue;
+
+	if (!w->shaded)
+	{
+	    if (w->attrib.map_state != IsViewable || !w->damaged)
+		continue;
+	}
+
+	if ((screen->windowOffsetX != 0 || screen->windowOffsetY != 0) &&
+	    !windowOnAllViewports (w))
+	{
+	    getWindowMovementForOffset (w, screen->windowOffsetX,
+					screen->windowOffsetY, &offX, &offY);
+
+	    vTransform = *transform;
+	    matrixTranslate (&vTransform, offX, offY, 0);
+	    (*screen->paintWindow) (w, &w->paint, &vTransform, w->clip,
+				    windowMask | PAINT_WINDOW_WITH_OFFSET_MASK);
+	}
+	else
+	{
+	    (*screen->paintWindow) (w, &w->paint, transform, w->clip,
+				    windowMask);
+	}
+    }
+
+    if (walk.fini)
+	(*walk.fini) (screen, &walk);
+
+    /* paint cursors */
+    for (c = screen->cursors; c; c = c->next)
+	(*screen->paintCursor) (c, transform, tmpRegion, 0);
+}
+
+void
+enableOutputClipping (CompScreen 	  *screen,
+		      const CompTransform *transform,
+		      Region		  region,
+		      CompOutput 	  *output)
+{
+    GLdouble h = screen->height;
+
+    GLdouble p1[2] = { region->extents.x1, h - region->extents.y2 };
+    GLdouble p2[2] = { region->extents.x2, h - region->extents.y1 };
+
+    GLdouble halfW = output->width / 2.0;
+    GLdouble halfH = output->height / 2.0;
+
+    GLdouble cx = output->region.extents.x1 + halfW;
+    GLdouble cy = (h - output->region.extents.y2) + halfH;
+
+    GLdouble top[4]    = { 0.0, halfH / (cy - p1[1]), 0.0, 0.5 };
+    GLdouble bottom[4] = { 0.0, halfH / (cy - p2[1]), 0.0, 0.5 };
+    GLdouble left[4]   = { halfW / (cx - p1[0]), 0.0, 0.0, 0.5 };
+    GLdouble right[4]  = { halfW / (cx - p2[0]), 0.0, 0.0, 0.5 };
+
+    glPushMatrix ();
+    glLoadMatrixf (transform->m);
+
+    glClipPlane (GL_CLIP_PLANE0, top);
+    glClipPlane (GL_CLIP_PLANE1, bottom);
+    glClipPlane (GL_CLIP_PLANE2, left);
+    glClipPlane (GL_CLIP_PLANE3, right);
+
+    glEnable (GL_CLIP_PLANE0);
+    glEnable (GL_CLIP_PLANE1);
+    glEnable (GL_CLIP_PLANE2);
+    glEnable (GL_CLIP_PLANE3);
+
     glPopMatrix ();
 }
 
+void
+disableOutputClipping (CompScreen *screen)
+{
+    glDisable (GL_CLIP_PLANE0);
+    glDisable (GL_CLIP_PLANE1);
+    glDisable (GL_CLIP_PLANE2);
+    glDisable (GL_CLIP_PLANE3);
+}
+
+#define CLIP_PLANE_MASK (PAINT_SCREEN_TRANSFORMED_MASK | \
+			 PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_MASK)
+
+void
+paintTransformedOutput (CompScreen		*screen,
+			const ScreenPaintAttrib *sAttrib,
+			const CompTransform	*transform,
+			Region			region,
+			CompOutput		*output,
+			unsigned int		mask)
+{
+    CompTransform sTransform = *transform;
+
+    if (mask & PAINT_SCREEN_CLEAR_MASK)
+	clearTargetOutput (screen->display, GL_COLOR_BUFFER_BIT);
+
+    screenLighting (screen, TRUE);
+
+    (*screen->applyScreenTransform) (screen, sAttrib, output, &sTransform);
+
+    if ((mask & CLIP_PLANE_MASK) == CLIP_PLANE_MASK)
+    {
+	screen->enableOutputClipping (screen, &sTransform, region, output);
+
+	transformToScreenSpace (screen, output, -sAttrib->zTranslate,
+				&sTransform);
+
+	glPushMatrix ();
+	glLoadMatrixf (sTransform.m);
+
+	paintOutputRegion (screen, &sTransform, region, output, mask);
+
+	glPopMatrix ();
+
+	screen->disableOutputClipping (screen);
+    }
+    else
+    {
+	transformToScreenSpace (screen, output, -sAttrib->zTranslate,
+				&sTransform);
+
+	glPushMatrix ();
+	glLoadMatrixf (sTransform.m);
+
+	paintOutputRegion (screen, &sTransform, region, output, mask);
+
+	glPopMatrix ();
+    }
+}
+
 Bool
-paintScreen (CompScreen		     *screen,
+paintOutput (CompScreen		     *screen,
 	     const ScreenPaintAttrib *sAttrib,
 	     const CompTransform     *transform,
 	     Region		     region,
-	     int		     output,
+	     CompOutput		     *output,
 	     unsigned int	     mask)
 {
-    static Region tmpRegion = NULL;
     CompTransform sTransform = *transform;
-    CompWindow	  *w;
 
     if (mask & PAINT_SCREEN_REGION_MASK)
     {
@@ -230,9 +415,9 @@ paintScreen (CompScreen		     *screen,
 	{
 	    if (mask & PAINT_SCREEN_FULL_MASK)
 	    {
-		region = &screen->outputDev[output].region;
+		region = &output->region;
 
-		(*screen->paintTransformedScreen) (screen, sAttrib,
+		(*screen->paintTransformedOutput) (screen, sAttrib,
 						   &sTransform, region,
 						   output, mask);
 
@@ -246,8 +431,8 @@ paintScreen (CompScreen		     *screen,
     }
     else if (mask & PAINT_SCREEN_FULL_MASK)
     {
-	(*screen->paintTransformedScreen) (screen, sAttrib, &sTransform,
-					   &screen->outputDev[output].region,
+	(*screen->paintTransformedOutput) (screen, sAttrib, &sTransform,
+					   &output->region,
 					   output, mask);
 
 	return TRUE;
@@ -262,90 +447,7 @@ paintScreen (CompScreen		     *screen,
     glPushMatrix ();
     glLoadMatrixf (sTransform.m);
 
-    if (mask & PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_MASK)
-    {
-	(*screen->paintBackground) (screen, region, 0);
-
-	/* paint all windows from bottom to top */
-	for (w = screen->windows; w; w = w->next)
-	{
-	    if (w->destroyed)
-		continue;
-
-	    if (!w->shaded)
-	    {
-		if (w->attrib.map_state != IsViewable || !w->damaged)
-		    continue;
-	    }
-
-	    (*screen->paintWindow) (w, &w->paint, &sTransform, region, 0);
-	}
-    }
-    else
-    {
-	int cnt = 0;
-
-	if (!tmpRegion)
-	{
-	    tmpRegion = XCreateRegion ();
-	    if (!tmpRegion)
-		return FALSE;
-	}
-
-	XSubtractRegion (region, &emptyRegion, tmpRegion);
-
-	/* paint solid windows */
-	for (w = screen->reverseWindows; w; w = w->prev)
-	{
-	    if (w->destroyed)
-		continue;
-
-	    if (!w->shaded)
-	    {
-		if (w->attrib.map_state != IsViewable || !w->damaged)
-		    continue;
-	    }
-
-	    if ((*screen->paintWindow) (w, &w->paint, &sTransform, tmpRegion,
-					PAINT_WINDOW_CLIP_OPAQUE_MASK))
-	    {
-		XSubtractRegion (tmpRegion, w->region, tmpRegion);
-
-		/* unredirect top most fullscreen windows. */
-		if (cnt == 0						  &&
-		    !REGION_NOT_EMPTY (tmpRegion)			  &&
-		    screen->opt[COMP_SCREEN_OPTION_UNREDIRECT_FS].value.b &&
-		    XEqualRegion (w->region, &screen->region))
-		{
-		    unredirectWindow (w);
-		}
-	    }
-
-	    /* copy region */
-	    XSubtractRegion (tmpRegion, &emptyRegion, w->clip);
-
-	    cnt++;
-	}
-
-	if (tmpRegion->numRects)
-	    (*screen->paintBackground) (screen, tmpRegion, 0);
-
-	/* paint translucent windows */
-	for (w = screen->windows; w; w = w->next)
-	{
-	    if (w->destroyed)
-		continue;
-
-	    if (!w->shaded)
-	    {
-		if (w->attrib.map_state != IsViewable || !w->damaged)
-		    continue;
-	    }
-
-	    (*screen->paintWindow) (w, &w->paint, &sTransform, w->clip,
-				    PAINT_WINDOW_CLIP_TRANSLUCENT_MASK);
-	}
-    }
+    paintOutputRegion (screen, &sTransform, region, output, mask);
 
     glPopMatrix ();
 
@@ -967,25 +1069,8 @@ drawWindow (CompWindow		 *w,
     if (!region->numRects)
 	return TRUE;
 
-    if (mask & PAINT_WINDOW_CLIP_OPAQUE_MASK)
-    {
-	if (w->attrib.map_state != IsViewable)
-	    return FALSE;
-
-	if (mask & PAINT_WINDOW_TRANSLUCENT_MASK)
-	    return FALSE;
-    }
-    else
-    {
-	if (w->attrib.map_state != IsViewable)
-	    return TRUE;
-
-	if (mask & PAINT_WINDOW_CLIP_TRANSLUCENT_MASK)
-	{
-	    if (!(mask & PAINT_WINDOW_TRANSLUCENT_MASK))
-		return FALSE;
-	}
-    }
+    if (w->attrib.map_state != IsViewable)
+	return TRUE;
 
     if (!w->texture->pixmap && !bindWindow (w))
 	return FALSE;
@@ -1013,20 +1098,35 @@ paintWindow (CompWindow		     *w,
 
     w->lastPaint = *attrib;
 
-    if (mask & PAINT_WINDOW_NO_CORE_INSTANCE_MASK)
+    if (w->alpha || attrib->opacity != OPAQUE)
+	mask |= PAINT_WINDOW_TRANSLUCENT_MASK;
+
+    w->lastMask = mask;
+
+    if (mask & PAINT_WINDOW_OCCLUSION_DETECTION_MASK)
     {
-	if (mask & PAINT_WINDOW_CLIP_OPAQUE_MASK)
+	if (mask & PAINT_WINDOW_TRANSFORMED_MASK)
+	    return FALSE;
+
+	if (mask & PAINT_WINDOW_NO_CORE_INSTANCE_MASK)
+	    return FALSE;
+
+	if (mask & PAINT_WINDOW_TRANSLUCENT_MASK)
+	    return FALSE;
+
+	if (w->shaded)
 	    return FALSE;
 
 	return TRUE;
     }
 
+    if (mask & PAINT_WINDOW_NO_CORE_INSTANCE_MASK)
+	return TRUE;
+
     initFragmentAttrib (&fragment, attrib);
 
-    if (w->alpha || fragment.opacity != OPAQUE)
-	mask |= PAINT_WINDOW_TRANSLUCENT_MASK;
-
-    if (mask & PAINT_WINDOW_TRANSFORMED_MASK)
+    if (mask & PAINT_WINDOW_TRANSFORMED_MASK ||
+        mask & PAINT_WINDOW_WITH_OFFSET_MASK)
     {
 	glPushMatrix ();
 	glLoadMatrixf (transform->m);
@@ -1034,7 +1134,8 @@ paintWindow (CompWindow		     *w,
 
     status = (*w->screen->drawWindow) (w, transform, &fragment, region, mask);
 
-    if (mask & PAINT_WINDOW_TRANSFORMED_MASK)
+    if (mask & PAINT_WINDOW_TRANSFORMED_MASK ||
+        mask & PAINT_WINDOW_WITH_OFFSET_MASK)
 	glPopMatrix ();
 
     return status;
